package jade.db;

/* This class is an adaptor object for communicating with the database.
   It handles all the messy socket stuff. */

import java.io.*;
import java.net.*;
import java.util.*;


class AceCache {

  /* max and min objects define how large the cache is */
  /* The cache will not grow beyond maxObjects size or
	 shrink below minObjects (except at the beginning while it
	 loads). */
  protected int maxObjects = 1500;
  protected int minObjects = 1000;

  protected int highWaterMark = Integer.MIN_VALUE;
  protected int lowWaterMark = Integer.MAX_VALUE;
  protected int objectsAdded = 0;
  protected int cacheHits = 0;
  protected int cacheMisses = 0;
  
  // private 
  private AceNode listTop = new AceNode(null);
  private AceNode listBottom = new AceNode(null,listTop);
  private AceNode reservedNode = new AceNode(null);
  private Hashtable dictionary;

  private static boolean debug = false ;

  // Constructor initializes various things.
  // We start out with a tiny linked list consisting of
  // two null linked pointers.  This grows by insertion.
  AceCache() { 
	dictionary = new Hashtable(minObjects);
  }  
  AceCache(int min, int max) {
	minObjects = min;
	maxObjects = max;
	dictionary = new Hashtable(minObjects);
  }  
  /* **************************************************** */
  // Protected functions.  Do not use.
  /* **************************************************** */

  // Add a new object to the queue
  protected synchronized void add (AceObject a) {
	// Create a new node object and place it at the
	// head of the queue.  Do not call this if the
	// object is already in the queue, because there
	// is no idiot checking at this level.
	AceNode n = new AceNode(a,listTop);
	objectsAdded++;

	// add object to the dictionary
	dictionary.put(hash(a.classe(),a.registration_id()),n);
	notifyAll();

	// do some garbage collection
	gc();
  }  
  protected void clearStats() {

//** What waterMarks for? **//
	  
	lowWaterMark = Integer.MAX_VALUE;
	highWaterMark = Integer.MIN_VALUE;
	objectsAdded = 0;
	cacheHits = 0;
	cacheMisses = 0;
  }    
  protected void dumpStats() {
	System.err.println("============= cache statistics ============");
	System.err.println("Low water mark (objects):  " + lowWaterMark);
	System.err.println("High water mark (objects): " + highWaterMark);
	System.err.println("Objects added:             " + objectsAdded);
	System.err.println("Cache hits:                " + cacheHits);
	System.err.println("Cache misses:              " + cacheMisses);
	if ((cacheHits + cacheMisses) > 0)
	  System.err.println("Hit rate (%):              " + (100.0*cacheHits/
							  (cacheHits+cacheMisses)));
  }  
  // Fetch an Ace object from the cache.  Return null if
  // not present.  If you pass 
  synchronized protected AceObject fetch (String classe, String name, boolean reserve) {
	if (debug)
	  System.err.println("AceCache.fetch() called for " + hash(classe,name) );
	AceNode n = lookup(classe,name);
	if (n == null) {
	  cacheMisses++;
	  // Reserve this name now.  The next lookup on this name will hang
	  // until a real ace object is placed in this slot.
	  if (reserve) reserve(classe,name);
	  return null;
	}
	cacheHits++;
	touch(n);
	if (debug)
	  System.err.println("AceCache.fetch() cache hit on " + n.object);
	return n.object;
  }  
  // Garbage collection.  Also implements the unfill() function.
  protected void gc() {
	int size = dictionary.size();
	if (size > highWaterMark)
	  highWaterMark = size;

	if (size > maxObjects) {
	  int objectsToRemove = size - minObjects - 2;
	  
	  while ((objectsToRemove > 0) && (listBottom.left != listTop) ) {
	AceNode a = listBottom.left;

	if (debug)
	  System.err.println("AceCache.gc(): uncaching object " + a.object);

	if (a.object == null)
	  System.err.println("AceCache.gc(): tried to delete a NULL ace object from the cache");
	else
	  a.object.unfill(); // suggest to the ace object that it should unfill() itself.

	dictionary.remove(hash(a.object.classe(),a.object.registration_id()));
	listBottom.left.delete();
	objectsToRemove--;
	  }

	}
	
	size = dictionary.size();
	if (size < lowWaterMark)
	  lowWaterMark = size;
  }  
  protected String hash (String classe, String name) {
	return "?" + classe + "?" + name + "?";
  }  
  protected AceNode lookup (AceObject a) {
	return lookup(a.classe(),a.registration_id());
  }  
  // Given a class and name, looks up an Ace object
  // in the dictionary and returns a pointer to it
  protected synchronized AceNode lookup (String classe, String name) {
	AceNode a = (AceNode) dictionary.get(hash(classe,name));
	if (debug) System.err.println("AceCache.lookup(): got " + a);
	while ( a == reservedNode ) {
	  try {
	if (debug) System.err.println("Waiting on a reserved node...");
	wait();
	a = (AceNode) dictionary.get(hash(classe,name));
	  } catch (InterruptedException e) {}
	}
	return a;
  }  
  // Add name to cache but don't allocate an entry.  We will wait
  // on reserved names until they are really allocated.
  synchronized protected void reserve (String classe, String name) {
	AceNode n = (AceNode)dictionary.get(hash(classe,name));
	if (n != null) return;
	if (debug) System.err.println("AceCache: reserving ?" + classe + "?"+name+"?");
	dictionary.put(hash(classe,name),reservedNode);
  }  
  // Move a previously-cached node to the head of the
  // queue
  protected void touch(AceNode n) {
	if (debug)
	  System.err.println("Touching " + n);
	n.insert(listTop);
  }  
  // Touch an object, moving it to the head of the queue.
  // If the object is not in the cache, we cache it now.
  synchronized void touch (AceObject a) {

	AceNode n = (AceNode) dictionary.get(hash(a.classe,a.registration_id())) ;
	// mieg, june 96: was:dictionary.get(hash(a.classe,a.name))

	// Object not in cache, so create it
	if ((n == null) || (n == reservedNode)) {
	  if (debug)
	System.err.println("Cache: adding " + a);
	  add(a);
	  return;
	}
	touch(n);
  }  
  // unreserve is intended to be used in the case that the object is not 
  // retrievable from the database.  Currently nobody calls this 
  // function.
  synchronized protected void unreserve (String classe, String name) {
	if (debug) System.err.println("AceCache: unreserving ?" + classe + "?"+name+"?");
	AceNode n = (AceNode)dictionary.get(hash(classe,name));
	if (n != reservedNode) return;
	dictionary.remove(hash(classe,name));
	notifyAll();
  }  
}