/*
 * SequenceView.java: A Sequence graphics view for chromosome and other sequence objects
 *
 * Created on December 4, 2000, 11:26 AM
 */

package jadex.swing.classView;

import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.jar.*;
import java.awt.event.*;
import javax.swing.*;

import jade.db.*;
import jadex.swing.sequence.*;

/**
 * A Sequence graphics view for chromosome and other sequence objects
 * @author  wgm
 * @version 1.0
 */
public class SequenceView extends ClassView {

    /** This is a URL address all XML files of chromosomes should be in. This line must be changed.  */
    public static String WORMBASE = "http://brie2.cshl.org:8081/applets/";
    private JFrame frame = null; // The JFrame holding all components: display, mapscale and others.
    
    private SequenceViewFactory viewFactory = null; // Set up all display environments.

    /** Creates new SequenceView */
    public SequenceView() {
        initialize();
    }

    /** Second constructor
     *@param name the chromosome name
     */
    public SequenceView(String name) {
        initialize();
        setSequence(name);
    }

    /** Initialize the layout */
    private void initialize() {
        frame = new JFrame();
        viewFactory = new SequenceViewFactory(frame);
    }

    /** Set the name of the chromosome to be displayed.
    * @param name the name of the chromosome to be displayed, e.g., Chromosome_I.
    */
    public void setSequence(String name) {
        frame.setTitle("Sequence View: " + name);
        
        InputStream input = null; // The source for XML files
        try {
            input = new FileInputStream(getFileName(name));
        }
        catch(FileNotFoundException e) { // If there are no files on the local machine,
                                        // Try find them on the wormbase.
            input = getInputStream(name);
        }
        if (input != null) {
            SequenceFetcher fetcher = new SequenceXMLFetcher(input);
            viewFactory.setSequenceFetcher(fetcher);
        }
    }

    /** Search a sequence object in the displayed chromosome.
    * @param name the name of the sequence object to be searched.
    */
    public void search(String name) {
        viewFactory.search(name);
    }
    /** Transform the chromosome name into a xml file name.
     * @param name the chromsome name
     * @return the xml file name
     */
    private String getFileName(String name) {
        StringBuffer buffer = new StringBuffer(name.toLowerCase());
        buffer.append(".xml");
        return buffer.toString();
    }

    /** Get the source of XML as inputstream
    * @param name the name of the chromosome to be displayed.
    * @return the inputstream the xml files will be from.
    */
    // This method must have to be changed!!!
    private InputStream getInputStream(String name) {
        JarURLConnection jarConnection = null;
        try {
            URL url = new URL("jar:" + WORMBASE + name + ".jar!/");
            jarConnection = (JarURLConnection) url.openConnection();           
            JarFile file = jarConnection.getJarFile();
            JarEntry entry = file.getJarEntry(name + ".xml");
            return file.getInputStream(entry);
            
        }
        catch(IOException e) {           
            System.err.println("SequenceViewApplet.getInputStream(): " + e);
            return null;
        } 
        catch(NullPointerException e1) {
            try {
                JarFile file = new JarFile(name + ".jar");
                JarEntry entry = file.getJarEntry(name + ".xml");
                return file.getInputStream(entry);
            }
            catch(IOException e2) {
                System.err.println("SequenceViewApplet.getInputStream(): " + e2);
                return null;
            }
        }
        finally {jarConnection = null;}
    }
    
    /** AceObject will not be supported anymore. However, this method is still 
     * used to integate the sequenceview into Java AceDB browser for the time being.
     * @param object the AceObject to be displayed.
     * @exception UnsupportedOperationException if this sequence view cannot be displayed.
     */
    public void setObject(AceObject object) throws UnsupportedOperationException {
        if (object.classe().equalsIgnoreCase("Sequence")) {
            String chromosome = getChromosome(object);
            if (chromosome != null) {
                setSequence(chromosome);
                search(object.name());
            }
            else {
                getViewFrame().setVisible(false);
                getViewFrame().dispose();
                throw new UnsupportedOperationException("SequenceView.SetObject(): Cannot find chromosome this sequence is in");
            }
        }
        else {
            getViewFrame().setVisible(false);
            getViewFrame().dispose();
            throw new UnsupportedOperationException("SequenceView.setObject(): Not a Sequence object");
        }
    }

    /** A helper method to find which chromosome this sequence is.
     * @param object the AceObject to be searched.
     * @return the chromosome name this sequence is. If not, return null.
     */
    private String getChromosome(AceObject obj) {
        String seqName = obj.name();
        // If this sequence object is one of chromosomes, just return its name.
        if (seqName.indexOf("CHROMOSOME") > -1) return obj.name();
        
        AceObject chr = null;
        String name = null;
        AceObject tmp = null;
        // Try to see if there is a Map information.
        tmp = obj.get("Map");
        if (tmp != null) {
            chr = tmp.right();
            name = parseChr(chr);
        }
        // If not, try to see there is an Interpolated_gMap
        else {
            tmp = obj.get("Interpolated_gMap");
            if (tmp != null) {
                chr = tmp.right();
                name = "CHROMOSOME_" + chr.name();
            }
            // If still not, try to see there is a chromosome can be found from source.
            else  name = searchChromosome(obj);
        }
        if (name != null) return name;
        else return null;
    }
    
    /** A utility to find which chromosome this sequence object is in. */
    private String searchChromosome(AceObject seq) {
        AceAccessor accessor = (AceAccessor) seq.accessor();
        AceObject chr = null;
        while (true) {
            AceObject source = seq.get("Source");
            if (source == null) return null;
            else chr = source.right();
            if (chr == null) break;
            if (chr.name.indexOf("CHROMOSOME") > -1) return chr.name;
            seq = accessor.fetch("Sequence", chr.name);
        }
        return null;
    }
    
    /** A helper to get the name of the chromosome based on the AceObject. */
    private String parseChr(AceObject chromosome) {
        String name = chromosome.name();
        int pos = name.indexOf("-");
        String map = name.substring(pos + 1);
        return "CHROMOSOME_" + map;
    }

    /** No aceobject anymore.
     * @return null
     */
    public AceObject getObject() {
        return null;
    }

    /** Get the JComponent holding this SequenceView.
    * @return the JComponent holding this SequenceView.
    */
    public JComponent getView() {
        return viewFactory.getSequenceDisplay();
    }

    /** Do nothing. Never call this method.
    */
    public void setView(JComponent comp) {
    }

    /** Override the super method. Get the JFrame displaying this SequenceView.
    * @return the JFrame displaying this SequenceView.
     */
    public JFrame getViewFrame() {
        return this.frame;
    }

    /** Clear up.
    */
    public void clear() {
        viewFactory.clear();
        frame.getContentPane().removeAll();
    }
    
    /** A test method. */
    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("Usage java SequenceView chromoseName");
            return;
        }
        SequenceView view = new SequenceView(args[0]);
        view.getViewFrame().addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        if (args.length == 2) {
            view.search(args[1]);
        }
    }
}