/** This is a control class for LocusDisplay. All control activities will
 *  be placed here.
 * Never attemp to put all functions into one class
 */

package jadex.swing.map;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.util.*;

/** This is a control class for LocusDisplay. All control activities will
 *  be placed here. It can listen to "zoom" property change event
 */
public abstract class DisplayControl implements PropertyChangeListener {
    public static int NUMBER_PER_COLUMN = 20; // The default number for showing in one column.
    public Color HIGHLIGHT_COLOR = Color.red;
    public Color NORMAL_COLOR = Color.black;

    private DisplayModel model; // model processing locus data
    private MapControl control; // The control for the whole MapView
    private MapRange map; // The whole MapRange of the shown chromosome
    private MapRange range; // Current maprange
    protected int showNumber = NUMBER_PER_COLUMN; // The locusUI number should be shown
    protected MapPoint[] tPoints; // Total MapPoints in this MapRange
    protected MapPoint[] sPoints; // Showing MapPoints in this MapRange
    protected LocusUI[] sLoci; // Loci will be shown in display

    // Searching pattern
    protected String pattern = null;
    protected String[] patterns = null; // for muli-search

    // This class can control the appearance of two knobs in locusdisplay
    private Knob topKnob = null;
    private Knob bottomKnob = null;

    /** Default constructor. */
    public DisplayControl() {
    }

    /** An overloaded constructor.
     * @param m the DisplayModel.
     */
    public DisplayControl(DisplayModel m) {
        setModel(m);
    }

    /** set knobs in this display: Assume all display have knobs.
     * @param top the top knob.
     * @param bottom the bottom knob.
     */
    public void setKnobs(Knob top, Knob bottom) {
        topKnob = top;
        bottomKnob = bottom;
    }

    /** DisplayModel can be set: A property of this class
     * @param m the DisplayModel.
     */
    public void setModel(DisplayModel m) {
        model = m;
        if (range != null) {
            tPoints = model.getMapPoints(range);
            resetUI();
        }
    }

    /** Get the DisplayModel this control controlled.
     * @return the model used in this Control.
     */
public DisplayModel getModel() { return model; }

    /** MapControl is always the central controller.
     * @param c the central controller for all zooming activities.
     */
    public void setMapControl(MapControl c) {
        this.control = c;
        map = control.getMap();
        range = map;
    }

    /** Get the central controller.
    * @return the MapControl used in this DisplayControl.
    */
public MapControl getMapControl() { return this.control; }

    /** PropertyChangeListener method: for zoom property fired by mapcontrol */
    public void propertyChange(PropertyChangeEvent e) {
        if (e.getPropertyName().equalsIgnoreCase("range")) {
            MapRange r = (MapRange) e.getNewValue();
            setMapRange(r);
        }
    }

    /** Do zooming.
    * @param r the maprange to be zoomed in.
    */
    public void zoom(MapRange r) {
        if (control != null) control.setRange(r);
        else setMapRange(r);
    }

    /** Set the total maprange.
    * @param t the total MapRange in this DisplayControl.
    */
    public void setTotalMapRange(MapRange r) {
        map = r;
        //range = r;
    }

    /** Get the total MapRange.
    * @return get the total MapRange used in this control.
    */
    public MapRange getTotalMapRange() {
        return map;
    }

    /** MapRange is set by this control class. 
    * @param r the MapRange will be be displayed.
    */
    public void setMapRange(MapRange r) {
        // If maprange is still the same, do nothing
        if (map == null) map = r; // Probably during initializing
        checkMapRange(r);
        if ((range != null) && range.equalsF(r)) return ;
        range = r;
        if (topKnob != null) checkKnobs();
        if (getDisplay() != null) getDisplay().setRange(r);
        if (model == null) return;

        // else get new mappoints, and set it in display
        tPoints = model.getMapPoints(r);
        //display.setTotalLocus(tPoints);
        resetUI();
        //if (pattern != null) doSearch(pattern);
        //else updateUI();
        //if (display.getPopup() != null) display.getPopup().checkShowMenu();
    }

    /** See if the maprange is out of the full scale. If yes, convert the ends */
    protected void checkMapRange(MapRange r) {
        try {
            float s = r.getStartF();
            float e = r.getEndF();
            if (s < map.getStartF()) r.setStartF(map.getStartF());
            if (e > map.getEndF()) r.setEndF(map.getEndF());
        }
        catch(UnknownPositionException e) {
            System.err.println("DisplayControl.checkMapRange(): " + e);
        }
    }

    /** Get the TotalShowNumber in this displayed maprange. 
    * @return the totalshowNumber in this displayed maprange.
    */
    public int getTotalShowNumber() {
        if (tPoints != null) return tPoints.length;
        else return 0;
    }

    /** Reset to the whole chromosome range. */
    public void reset() {
        if (control != null) control.reset();
        else {
            if (map != null) setMapRange(map);
        }
    }

    /** Show more. */
    public void showMore() {
        setShowNumber(showNumber + NUMBER_PER_COLUMN);
    }

    /** Show less. */
    public void showLess() {
        int size = showNumber - NUMBER_PER_COLUMN;
        if (size < 0) size = 0;
        setShowNumber(size);
    }

    /** Show reset. Show the default number of MapPoints. */
    public void showReset() {
        setShowNumber(NUMBER_PER_COLUMN);
    }

    /** Showing mappoint with the total number of number. 
    * @param number the showing number of mappoints.
    */
    public void setShowNumber(int number) {
        this.showNumber = number;
        resetUI();
    }

    /** Moving up the displayed range by knob. */
    public void movingUp() {
        if (control != null) control.movingUp(); // If there is a MapControl, ask it to do moving
        else { // else, has to do by itself
            try {
                float s = range.getStartF();
                float e = range.getEndF();
                float step = range.getExtendF() / 3.0f;
                MapRange rNew = new MapRange(s + step, e + step);
                setMapRange(rNew);
            }
        catch (UnknownPositionException e) { System.err.println(e); }
        }
    }

    /** Moving down the displayed range by knob. */
    public void movingDown() {
        if (control != null) control.movingDown(); // Same as moving down
        else {
            try {
                float s = range.getStartF();
                float e = range.getEndF();
                float step = range.getExtendF() / 3.0f;
                MapRange rNew = new MapRange(s - step, e - step);
                setMapRange(rNew);
            }
        catch (UnknownPositionException e) { System.err.println(e); }
        }
    }

    /** Check the arrow of two knobs. */
    public void checkKnobs() {
        if ((map != null) && (range != null)) {
            try {
                if (range.getStartF() > map.getStartF()) bottomKnob.setDirection(false);
                else bottomKnob.setDirection(true);
                if (range.getEndF() < map.getEndF()) topKnob.setDirection(false);
                else topKnob.setDirection(true);
            }
            catch (UnknownPositionException e) {
                System.err.println("DisplayControl:checkKnobs(): " + e);
            }
        }
    }

    /** Display should be provided by subclass. 
    * @return a MapPointDisplay showing mappoint data.
    */
    public abstract MapPointDisplay getDisplay();

    /** Search the loci with string pattern in their names. 
    * @param pat the searched key.
    */
    /** Consider to change this method in the future. Probably a new SelectionModel 
     * class is needed for search purpose.
    */
    public void doSearch(String pat) {
        // Clear up the latest search
        //deSearch();
        String pat1 = pat.trim();
        if (!pat1.equalsIgnoreCase(pattern)) { // This is a new search
            pattern = pat1;
            if (pattern.indexOf(" ") > 0) { // This is multi-searching
                StringTokenizer tokenizer = new StringTokenizer(pattern);
                patterns = new String[tokenizer.countTokens()];
                int i = 0;
                while (tokenizer.hasMoreTokens()) {
                    patterns[i] = tokenizer.nextToken();
                    i++;
                }
            }
            else patterns = null;
        }

        // Search the shown loci first: use an ineffcient linear search temporarily
        if ((sLoci != null) && (sLoci.length > 0)) {
            for (int i = 0; i < sLoci.length; i++) {
                String name1 = sLoci[i].getName();
                if (patterns != null) {
                    for (int j = 0; j < patterns.length; j++) {
                        if (name1.indexOf(patterns[j]) > -1) {
                            sLoci[i].setForeOutColor(HIGHLIGHT_COLOR);
                            break; //break this loop if found
                        }
                    }
                }
                else if (name1.indexOf(pattern) > -1) sLoci[i].setForeOutColor(HIGHLIGHT_COLOR);
            }
        }

        // However, we still have to search unshown loci
        if (sLoci.length < tPoints.length) {

            java.util.List shown = new Vector(Arrays.asList(sPoints));
            java.util.List totalP = new Vector(Arrays.asList(tPoints));
            try {
                totalP.removeAll(shown);
            }
        catch(UnsupportedOperationException e) { System.err.println("LocusControl: " + e); }

            java.util.List sP = new Vector(Arrays.asList(sPoints));
            java.util.List sL = new Vector(Arrays.asList(sLoci));

            for (Iterator i = totalP.iterator(); i.hasNext(); ) {
                MapPoint p = (MapPoint) i.next();
                String name = p.getName();
                if (patterns != null) {
                    for (int j = 0; j < patterns.length; j++) {
                        if (name.indexOf(patterns[j]) > -1) {
                            addLocusUI(sP, sL, p);
                            break; //break this loop if found
                        }
                    }
                }
                else if (name.indexOf(pattern) > -1) {
                    addLocusUI(sP, sL, p);
                }
            }
            sPoints = (MapPoint[]) sP.toArray(new MapPoint[0]);
            sLoci = (LocusUI[]) sL.toArray(new LocusUI[0]);
        }
        updateUI();
    }

    // Just a helper method
    private void addLocusUI(java.util.List l1, java.util.List l2, MapPoint p) {
        l1.add(p);
        LocusUI ui = new LocusUI(p);
        ui.setForeOutColor(HIGHLIGHT_COLOR);
        l2.add(ui);
    }

    /** Remove all label and close all extra locusUI. */
    public void deSearch() {
        if (pattern == null) return ; // Do nothing, no searching happened.
        pattern = null;

        if ((sLoci != null) && (sLoci.length > 0)) {
            for (int i = 0; i < sLoci.length; i++) {
                sLoci[i].setForeOutColor(NORMAL_COLOR);
            }
        }
        updateUI();
    }

    /** Get the number of shown LocusUI.
    * @return the number of shown LocusUI.
    */
    public int getShowNumber() {
        if (sLoci != null) return sLoci.length;
        else return 0;
    }

    /** UI should be reset after total points or showing points in range have 
     * been changed.
    */
    protected abstract void resetUI();

    /** Update UI after search or desearch */
    public abstract void updateUI();
}