/*
 * DisplayModel.java: The model used for all kinds of displays
 *
 * Created on July 27, 2000, 9:53 AM
 */

package jadex.swing.map;

import java.util.*;
import jadex.swing.map.MapRange;

/**
 * This is a datamodel used for both GeneticMap and SequenceMap displays.
 * @author  wgm
 * @version 1.0
 */
public class DisplayModel extends Object {

    private List all = null; // all mappoints can be displayed.
    private Vector points = null; // currently displayed mappoints.
    private Comparator comp = null; // a comparator for sorting mappoints.
    // this comparator for sorting mappoints by position errors.
    private Comparator errComp = new Comparator() {
        public int compare(Object o1, Object o2) {
            MapPoint p1 = (MapPoint) o1;
            MapPoint p2 = (MapPoint) o2;
            String n1 = p1.getName();
            String n2 = p2.getName();
            Float e1 = new Float(p1.getError());
            Float e2 = new Float(p2.getError());
            int rtn = e1.compareTo(e2);
            return (rtn!= 0 ? rtn : n1.compareTo(n2));

            //return n1.compareTo(n2);
        }

        public boolean equals(Object o1, Object o2) {
            MapPoint p1 = (MapPoint) o1;
            MapPoint p2 = (MapPoint) o2;
            String name1 = p1.getName();
            String name2 = p2.getName();
            return name1.equalsIgnoreCase(name2);
        }
    };

    /** Creates new DisplayModel */
    public DisplayModel() {
        all = new ArrayList();
        comp = new Comparator() {
            public int compare(Object o1, Object o2) {
                MapPoint p1 = (MapPoint) o1;
                MapPoint p2 = (MapPoint) o2;
                Float pos1 = null;
                Float pos2 = null;
                try {
                    pos1 = new Float(p1.getPosition());
                    pos2 = new Float(p2.getPosition());
                }
            catch (UnknownPositionException e) { System.err.println(e); }
                return pos2.compareTo(pos1);
            }
            public boolean equals(Object o1, Object o2) {
                MapPoint p1 = (MapPoint) o1;
                MapPoint p2 = (MapPoint) o2;
                String name1 = p1.getName();
                String name2 = p2.getName();
                return name1.equalsIgnoreCase(name2);
            }
        };
    }

    /** Add a MapPoint.
    * @param p a MapPoint to be added.
    */
    public void addPoint(MapPoint p) {
        all.add(p);
    }

    /** Vector is a collection of MapPoint to be displayed.
    * @param v the MapPoint data that can be displayed.
    * @deprecate use another {@link setData(List) setData(List)}.
    */
    public void setData(Vector v) {
        all = new ArrayList(v);
    }

    /** Set the total displayable MapPoint data.
    * @param l a list holding all MapPoints.
    */
    public void setData(List l) {
        all = l;
    }

    /** Set the total displayable MapPoint data from a MapPoint array.
    * @param points a MapPoint array holding all data.
    */
    public void setData(MapPoint[] points) {
        all = Arrays.asList(points);
    }

    /** Get all MapPoint in an array.
    * @return a MapPoint array holding all MapPoints.
    */
    public MapPoint[] getData() {
        return (MapPoint[]) all.toArray(new MapPoint[0]);
    }

    /** Get the MapPointn with the number of size.
    * @param size the number of the MapPoints to be got.
    */
    //This method will be changed!
    public MapPoint[] getMapPoints(int size) {
        List copy = new ArrayList(all);
        List shown = copy.subList(0, size);
        return (MapPoint[]) shown.toArray(new MapPoint[0]);
    }

    /** Giving a maprange, get all mappoints in this range.
    * @param r the MapRange limiting the query.
    * @return a MapPoint array in the MapRange r.
    */
    public MapPoint[] getMapPoints(MapRange r) {
        List list = new ArrayList(all);
        try {
            float start = r.getStartF();
            float end = r.getEndF();
            for (ListIterator i = list.listIterator(); i.hasNext(); ) {
                MapPoint p = (MapPoint) i.next();
                float pos = p.getPosition();
                if ((pos <= start) || (pos >= end)) i.remove();
            }
        }
    catch (UnknownPositionException e) { System.err.println(e); }
        return (MapPoint[]) list.toArray(new MapPoint[0]);
    }

    /** Giving a maprange and showing number, get the mappoints.
    * @param r the MapRange limiting the query.
    * @param number the number of the MapPoints to be queried.
    * @return a MapPoint array in the MapRange.
    */
    public MapPoint[] getMapPoints(MapRange r, int number) {
        MapPoint[] total = getMapPoints(r);
        int size = total.length < number ? total.length : number;
        MapPoint[] rtn = new MapPoint[size];
        for (int i = 0; i < size; i++)
        rtn[i] = total[i];
        return rtn;
    }

    /** Set the comparator for sorting the MapPoints.
    * @param comp the comparator for sorting the MapPoints.
    */
    public void setComparator(Comparator comp) {
        this.comp = comp;
    }

    /** Get the comparator for sorting the MapPoints.
    * @return the Comparator for sorting the MapPoints.
    */
public Comparator getComparator() { return this.comp; }

    /** Sort all MapPoints. */
    public void sort() {
        Collections.sort(all, comp);
    }

    /** Sort all MapPoints by position errors. */
    public void sortByError() {
        sort(errComp);
    }

    /** Sort all MapPoints by a specified comparator.
    * @param comp a Comparator used for sorting all MapPoints.
    */
    public void sort(Comparator comp) {
        Collections.sort(all, comp);
    }

    /** A utility for sorting a MapPoint array using a Comparator.
    * @param points the MapPoint array to be sorted.
    * @param comp the Comparator for sorting the MapPoint arrays.
    */
    public void sort(MapPoint[] points, Comparator comp) {
        List l = Arrays.asList(points);
        Collections.sort(l, comp);
    }

    /** Sort a MapPoint array by using defalt comparator.
    * @param points a MapPoint array to be sorted.
    */
    public void sort(MapPoint[] points) {
        sort(points, this.comp);
    }

    /** Sort a MapPoint list by using the default Comparator.
    * @param points a MapPoint List to be sorted.
    */
    public void sort(List points) {
        Collections.sort(points, this.comp);
    }

    /** Distribute all MapPoints.
    * @param rel the ratio between the biological distance (e.g., genetic or 
    * sequence distances) and graphic distance.
    */
    public Vector assignPoints(float rel) {
        points = new Vector();
        assignPoints(getData(), rel);
        return points;
    }
    
    /** Distribute a MapPoint array.
    * @param l the data to be distributed.
    * @param rel the ratio.
    */
    protected void assignPoints(MapPoint[] l, float rel) {
        Vector left = new Vector();
        Vector added = new Vector();
        float pos = Float.MAX_VALUE;
        for (int i = 0; i < l.length; i++) {
        try {pos = l[i].getPosition();}
        catch (UnknownPositionException e) { System.err.println(e); }
            if (checkAdding(added, pos, rel)) {
                added.addElement(l[i]);
            }
            else {
                left.addElement(l[i]);
            }
        }
        points.addElement(added);
        if (left.size() == 0) return;
        else {
            int size = left.size();
            MapPoint[] p = new MapPoint[size];
            for (int i = 0; i < size; i++)
            p[i] = (MapPoint) left.elementAt(i);
            assignPoints(p, rel);
        }
    }

    //Checking two points' distance to determine if loci can be added.
    private boolean checkAdding(Vector p, float pos, float rel) {
        int size = p.size();
        MapPoint point = null;
        for (int i = 0; i < size; i++) {
            point = (MapPoint) p.elementAt(i);
            float position = Float.MAX_VALUE;
        try { position = point.getPosition();}
        catch (UnknownPositionException e) { System.out.println(e); }
            float diff = Math.abs((pos - position) * rel);
            if (diff <= 2.5f) return false;
        }
        return true;
    }
}