/** This class is used to hold all intervalPanel, and can be displayed in
 *  the mapview
 */

package jadex.swing.map;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;

/** A customized JPanel is used to hold all IntervalPanel, and can be displayed in the
 * MapView.
 * @author wgm
 * @version 1.0
 */
public class IntervalContainer extends JPanel implements PropertyChangeListener, MouseListener, MapZoom {
    private Vector panels = null; // Total number of intervalPanels
    private Vector intervals = null; // Total number of intervalUI
    private float extend = 0.0f;
    
    private MapRange range = null; // The currently displayed MapRange.
    private JPopupMenu popup = null; // Popup menu is used to control the dispaly of this container

    private MapControl control = null; // control is the control of this view

    /** Default constructor */
    public IntervalContainer() {    
        initialize();
    }

    /** Another constructor with maprange as the argument. 
    * @param scale the MapRange to be displayed.
    */
    public IntervalContainer(MapRange scale) {
        this.range = scale;
        initialize();
    }

    // All setting up staff are here
    private void initialize() {
        addMouseListener(this);
        setLayout(new GridLayout(1, 0));
        IntervalPanel p = new IntervalPanel(range);
        panels = new Vector();
        intervals = new Vector();
        addIntervalPanel(p);
        ToolTipManager.sharedInstance().setInitialDelay(100);
        ToolTipManager.sharedInstance().setDismissDelay(10000);
        ToolTipManager.sharedInstance().setReshowDelay(500);
        setPopup();
    }

    /** Set up a JPopupMenu. */
    private void setPopup() {
        JPopupMenu pop = new JPopupMenu();
        JMenuItem zoomOut = new JMenuItem("Zoom Out");
        zoomOut.setFont(new Font("Dialog", 0, 11));
        ActionListener a1 = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (control != null) control.reset();
            }
        };
        zoomOut.addActionListener(a1);

        JMenuItem clear = new JMenuItem("Deselect");
        clear.setFont(new Font("Dialog", 0, 11));
        clear.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int size = panels.size();
                IntervalPanel p = null;
                for (int i = 0; i < size; i++) {
                    p = (IntervalPanel) panels.elementAt(i);
                    p.resetBackground();
                }
            }
        });

        JMenuItem search = new JMenuItem("Search");
        search.setFont(new Font("Dialog", 0, 11));
        search.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String str = JOptionPane.showInputDialog(
                IntervalContainer.this, "Search for ...", "Interval Search", JOptionPane.INFORMATION_MESSAGE);
                if ((str != null) && (str.length() != 0)) doSearch(str);
            }
        });

        pop.add(zoomOut);
        pop.add(new JSeparator());
        pop.add(clear);
        pop.add(new JSeparator());
        pop.add(search);

        setPopup(pop);
    }

    /** Search Intervals which names containing s.
    * @param s the search key.
    */
    public void doSearch(String s) {
        int size = intervals.size();
        IntervalUI ui = null;
        String name = null;
        String str = s.toLowerCase();
        for (int i = 0; i < size; i++) {
            ui = (IntervalUI) intervals.elementAt(i);
            name = ui.getName().toLowerCase();
            if (name.indexOf(s) > -1) {
                ui.setBackground(Color.yellow);
            }
        }
    }

    /** Set up a JOpoupMenu.
    * @param pop the JPopupMenu to be used by this container.
    */
    public void setPopup(JPopupMenu pop) {
        this.popup = pop;
    }

    /** Individual IntervalPanel can be added into this container.
    * @param p a IntervalPanel to be added.
    */
    public void addIntervalPanel(IntervalPanel p) {
        panels.addElement(p);
        this.add(p);
    }

    /** Individual IntervalUI can be added into this container.
    * @param intervalUI an IntervalUI to be added.
    * @return true, if added.
    */
    public boolean addInterval(IntervalUI intervalUI) {

        intervals.add(intervalUI);

        IntervalPanel p = null;
        boolean added = false;
        int i = 0;
        int size = panels.size();
        while (i < size) {
            p = (IntervalPanel) panels.elementAt(i);
            if (p.checkAdding(intervalUI)) {
                p.addInterval(intervalUI);
                added = true;
                break;
            }
            i ++;
        }
        if (!added) {
            p = new IntervalPanel(range);
            p.addInterval(intervalUI);
            addIntervalPanel(p);
            added = true;
        }

        return added;
    }

    /** Property change listener for range property. */
    public void propertyChange(PropertyChangeEvent p1) {
        if (p1.getPropertyName().equalsIgnoreCase("range")) {
            MapRange r = (MapRange) p1.getNewValue();
            int size = panels.size();
            for (int i = 0; i < size; i++) {
                IntervalPanel p = (IntervalPanel) panels.elementAt(i);
                p.setRange(r);
            }
        }
    }

    // a test method
    public static void main (String[] args) {
        JFrame f = new JFrame("IntervalContainer Test");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        IntervalContainer intervals = new IntervalContainer(new MapRange(0.0f, 50.0f));
        IntervalUI i1 = new IntervalUI(2.0f, 20.0f, "Interval1");
        IntervalUI i2 = new IntervalUI(21.3f, 35.0f, "Interval2");
        IntervalUI i3 = new IntervalUI(10.0f, 25.9f, "Interval3");
        IntervalUI i4 = new IntervalUI(40.0f, 50.0f, "Interval4");
        IntervalUI i5 = new IntervalUI(45.0f, 50.0f, "Interval5");
        intervals.addInterval(i1);
        intervals.addInterval(i2);
        intervals.addInterval(i3);
        intervals.addInterval(i4);
        intervals.addInterval(i5);
        //intervals.showInterval();
        f.getContentPane().add(intervals, "Center");
        f.setSize(150, 200);
        f.setVisible(true);
    }

    // All mouse listener methods.
    /** For popup pop.
    */
    public void mouseReleased(final java.awt.event.MouseEvent p1) {
        if ((p1.isPopupTrigger()) && (popup != null)) {
            popup.show(this, p1.getX(), p1.getY());
        }
    }
    /** Do nothing. */
    public void mouseClicked(final java.awt.event.MouseEvent p1) {
    }
    /** Do nothing. */
    public void mouseEntered(final java.awt.event.MouseEvent p1) {
    }
    /** Showing popup. */
    public void mousePressed(final java.awt.event.MouseEvent p1) {
        if ((p1.isPopupTrigger()) && (popup != null)) {
            popup.show(this, p1.getX(), p1.getY());
        }
    }
    /** Do nothing. */
    public void mouseExited(final java.awt.event.MouseEvent p1) {
    }

    /** Expand or shrink on the horizontal direction. 
    * @param step the zooming step.
    */
    public void horizontalZoom(int step) {
        int size = panels.size();
        for (int i = 0; i < size; i++) {
            IntervalPanel p = (IntervalPanel) panels.elementAt(i);
            //p.setPreferredSize(new Dimension(p.getWidth() + step, 500));
            p.setSize(new Dimension(p.getWidth() + step, 500));
        }
        doLayout();
        invalidate();
    }

    /** No vertical zooming temporarily. */
    public void verticalZoom(int step) {
    }

    /** Set the mapcontrol for all intervalUI.
    * @param c the MapControl used for all IntervalPanel.
    */
    public void setMapControl(MapControl c) {
        control = c;
        IntervalPanel p = null;
        for (int i = 0; i < panels.size(); i++) {
            p = (IntervalPanel) panels.elementAt(i);
            p.setMapControl(c);
        }
    }
}