/** This class is used to hold intervalUIs in a vertical way
 */

package jadex.swing.map;

import javax.swing.*;
import java.util.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This customized JPanel is used to hold IntervalUIs in a vertical way.
 * @author wgm
 * @version 1.0
 */

public class IntervalPanel extends JPanel {
    private Vector intervals = null; // All intervalUIs in this vector
    private MapRange range = null; // range of this intervalPanel
    // These two variables are used to switch the coordinate systems between range and true one
    private float extend;
    private float offset;

    /** Default constructor. */
    public IntervalPanel() {
        intervals = new Vector();
        this.setLayout(null);
        this.setPreferredSize(new Dimension(15, 400));
    }

    /** Another constructor with maprange as its argument. 
    * @param scale the MapRange for this display.
    */
    public IntervalPanel(MapRange scale) {
        this();
        setRange(scale);
    }

    /** Set the MapControl for this IntervalPanel. The MapControl is used to coordinate
    * all zooming activities in the MapView.
    * @param c the MapControl to be used in this IntervalPanel.
    */
    public void setMapControl(MapControl c) {
        IntervalUI ui = null;
        for (int i = 0; i < intervals.size(); i++) {
            ui = (IntervalUI) intervals.elementAt(i);
            ui.setMapControl(c);
        }
    }

    /** Set the displaying MapRange.
    * @param r the MapRange to be displayed.
    */
    public void setRange(MapRange r) {
        this.range = r;
        try {
            extend = range.getExtendF();
            //if (extend < 0.001f) extend = 0.001f; // Control the bottom of zooming.
            offset = range.getEndF();
        }
        catch (UnknownPositionException e) {
            System.out.println("Position don't know: " + e);
        }
        repaint();
    }

    /** Individual IntervalUI can be added by this method.
    * @param interval an Indicidual IntervalUI to be added.
    */
    public void addInterval(IntervalUI interval) {
        intervals.addElement(interval);
        add(interval);
        interval.setBounds(0, 0, 10, 40); // Setting an original position of an interval
    }

    /** Check if IntervalUI can be added onto this panel.
    * @param iv an IntervalUI to be added.
    * @return true, this iv can be added.
     */
    public boolean checkAdding(IntervalUI iv) {
        boolean canBeAdded = true;

        IntervalUI ui = null;
        int i = 0;
        int size = intervals.size();
        while ( i < size) {
            ui = (IntervalUI) intervals.elementAt(i);
            if (ui.intersect(iv)) {
                canBeAdded = false;
                break;
            }
            i ++;
        }
        return canBeAdded;
    }

    /** GUI painting. */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g;
        Dimension2D d = (Dimension2D) getSize();

        float w = (float) d.getWidth();
        float h = (float) d.getHeight();
        float rel = h / extend;
        if (rel > 1000.0f) rel = 1000.0f; //Control the bottom of zooming.

        g2d.setColor(Color.white);
        g2d.fill(new Rectangle2D.Float(0.0f, 0.0f, (float) w, (float) h));

        for (int i = 0; i < intervals.size(); i++) {
            IntervalUI ui = (IntervalUI) intervals.elementAt(i);
            //float rel = (float) (h / extend);
            float s = (offset - ui.getStart()) * rel;
            float e = (offset - ui.getEnd()) * rel;
            ui.setBounds(0, (int) e, (int) w, (int) (s-e));
            //System.out.println("PaintComponet in IntervalPanel: " + s + "  " + e + " rel: " + rel);
        }
    }

    /** Reset the background of each intervalUI to black. */
    public void resetBackground() {
        int size = intervals.size();
        IntervalUI ui = null;
        for (int i = 0; i < size; i++) {
            ui = (IntervalUI) intervals.elementAt(i);
            ui.resetBackground();
        }
    }
}