/*
 * LocusDisplay.java: A locus display
 *
 * Created on July 27, 2000, 10:18 AM
 */

package jadex.swing.map;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;

import jadex.swing.util.*;
import jade.db.*;

/**
 * An implementation of MapPointDisplay for displaying Loci.
 * @author  wgm
 * @version 1.0
 */
public class LocusDisplay extends LocusShow implements MapPointDisplay, MouseListener, Zoomable {

    private MapRange range = null; // Changing range used to set the showing range.
    private MapRange map = null; // Total range of the chromosome.
    private LocusControl control = null; // Control almost all activites

    private MapPoint[] loci = null; // Total loci in the chromosome
    private MapPoint[] sTotal = null; // Total loci in a specific range
    private LocusUI[] lUI = null; // LocusUI will be shown in this displayy

    private Vector points = null; 
    private final float REL = 6.0f;

    // variables for switch between positions and coordinates of loci
    private float yPadding = 10.0f;
    private final float xPadding = 5.0f;
    private float rightP;
    private float offset, rel;
    
    // These variables for zooming
    private Rectangle2D.Float zoom = null; // The zooming region
    private float inX, inY, outX, outY; // The four corner of the zoom
    private Magnifier mag = null; // Magnifier class for zooming
    
    private boolean cShown = true; // A flag to control if the connecting lines should be displayed.
    private String mouseInName = null; // The name of LocusUI where mouse is in.

    private JPopupMenu pop2 = null; // A JPopupMenu

    // Two LocsuKnobs for controlling moving up and down
    private Knob topKnob = null;
    private Knob bottomKnob = null;

    //Color mode
    private Color pColor = Color.orange; // point color
    private Color vColor = Color.gray; // vertical line color
    private Color hColor = Color.black; // horizontal line color
    private Color cColor = Color.lightGray; // connecting line color
    private Color bColor = Color.white; // background color
    protected Color nColor = Color.black; // Locus name color (label)
    protected Color highLightColor = Color.blue; // Selected locus' background

    // Locus layout for controlling
    private LocusLayout layout = null; // default layout
    private LocusLayout columnLayout = null;
    private LocusLayout smartLayout = null;

    private MiniMapCartoon mini = null;

    /*
    
    private class LineMouse extends MouseAdapter {
        LocusUI ui = null;
        LocusDisplay display = null;

        LineMouse(LocusUI ui, LocusDisplay display) {
            this.ui = ui;
            this.display = display;
        }

        public void mouseEntered(MouseEvent e) {
            mouseInName = ui.getName();
            layout.setMouseInName(mouseInName);
            display.repaint();
        }
        public void mouseExited(MouseEvent e) {
            mouseInName = null;
            layout.setMouseInName(null);
            display.repaint();
        }
    }; */

    /** This constructor can only be used by classes in the same package */
    protected LocusDisplay() {
        initialize();
    }

    /** Initialize the total range by the constructor.
    * @param r the total MapRange to be displayed.
    */
    public LocusDisplay(MapRange r) {
        map = r;
        range = r;
        control = new LocusControl(this);
        control.setMapRange(range);
        mini = new MiniMapCartoon(map);
        mini.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
        initialize();
    }

    // All GUI initializing here
    private void initialize() {
        setPreferredSize(new Dimension(1600, 400));
        setLayout(null);
        JPopupMenu pop = new DisplayPopup(this, control, "locus");
        setPopup(pop);
        addMouseListener(this);
        columnLayout = new LocusLayout(yPadding, LocusControl.NUMBER_PER_COLUMN, cShown, cColor);
        smartLayout = new SmartLocusLayout(yPadding, LocusControl.NUMBER_PER_COLUMN, cShown, cColor);
        layout = columnLayout;
        setKnobs();
        mag = new Magnifier(this);
    }

    /** set locus model, which is the data class of locus display.
    * @param m the DataModel to for this display.
    */
    public void setModel(DisplayModel m) {
        control.setModel(m);
        loci = m.getData();
        points = m.assignPoints(REL);
        //repaint();
    }

    /** Get the LocusControl used by this display.
    * @return the LocusControl used by this Display.
    */
public LocusControl getLocusControl() { return control; }

    // Create the customized cursor only in application
    private Cursor createCursor() {
        Cursor c = null;
        try {
            Toolkit kit = Toolkit.getDefaultToolkit();
            String sep = File.separator;
            final	String path = new String("jadex" + sep + "swing" + sep + "map" + sep);
            ImageIcon icon = new ImageIcon(path + "ZoomIn.gif");
            Image img = icon.getImage();

            c = kit.createCustomCursor(img, new Point(8, 8), "CUSTOMER_CURSOR");
            //System.out.println(kit.getBestCursorSize(4, 4));
        }
        catch (Exception e) {
            //System.err.println(e);
            c = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
        }
        return c;
    }

    // set two knobs
    private void setKnobs() {
        topKnob = new UpKnob(control);
        bottomKnob = new DownKnob(control);
        control.setKnobs(topKnob, bottomKnob);
    }

    /** Set up a JPopupMenu for controlling this locus display.
    */
    public void setPopup(JPopupMenu pop) {
        pop2 = pop;
    }

    /** Get the JPopupMenu used by this Display.
    */
    public DisplayPopup getPopup() {
        return (DisplayPopup) pop2;
    }

    /** Set the Layout for the name labels for LocusUI.
    * @param l a LocusLayout will be used for layout names for the displayed LocusUIs.
    */
    public void setNameLayout(LocusLayout l) {
        this.layout = l;
        layout.setLocusUI(lUI);
        repaint();
    }

    /** Set up a NameLayout by their names. 
    * @param name the name of the LayoutModel. Now only two names are supported, 
    * columnLayout and smartLayout.
    */
    public void setNameLayout(String name) {
        if (name.equalsIgnoreCase("columnLayout")) {
            setNameLayout(columnLayout);
        }
        else if (name.equalsIgnoreCase("smartLayout")) setNameLayout(smartLayout);
    }

    /** The connecting line can be turn on or off by this method. 
    * @param b true for turning on, false for turning off the connecting lines.
    */
    public void setConnectShown(boolean b) {
        cShown = b;
        //layout.setConnect(b);
        columnLayout.setConnect(b);
        smartLayout.setConnect(b);
        repaint();
    }

    /** Get the flag of connecting lines.
    * @return true if connecting lines are on.
    */
public boolean isConnectShown() { return cShown; }

    /** Set up the displaying MapRange.
    * @param r the MapRange to be displayed.
    */
    public void setRange(MapRange r) {
        this.range = r;
        if (mini != null) mini.setRange(r);
    }

    /** Get the displaymodel.
    * @return the DisplayModel used by this Display.
    */
public DisplayModel getModel() { return control.getModel(); }

    /** loci will be shown by this method. 
    * @param ui the LocusUI data to be displayed.    
    */
    protected void setLocusUI(LocusUI[] ui) {
        lUI = ui;
        this.removeAll();
        add(mini);
        mini.setBounds(0, 0, 20, 60);

        // If nothing in locus, just return
        if ((lUI != null) && (lUI.length > 0)) {

            for (int i = 0; i < lUI.length; i++) {
                add(lUI[i]);
                lUI[i].setBounds(0, 0, 40, 10); //Have to set the original boounds
                lUI[i].addMouseListener(new LineMouse(lUI[i], this));
            }
        }
        layout.setLocusUI(lUI);
    }

    /** Get the Layout for Names.
    * @return the namelayout currently used in this Dislay.
    */
    public LocusLayout getNameLayout() {
      return layout;
    }
    
    /** Set the name for lightening up label and line when mouse is in.
    * @param name the name of the LocusUI where mouse is in.
    */
    public void setMouseInName(String name) {
      this.mouseInName = name;
      repaint();
    }
    
    /** Set the total loci in a specific range. 
    * @param t the total MapPoints in the specified Range.
    */
    protected void setTotalLocus(MapPoint[] t) {
        sTotal = t;
    }

    /** Get the position of the Chromosome line. This method is for implementation
     * of Zoomable.
    * @return the position fo the chromosome line.
    */
    public float getVerticalPosition() {
        return rightP;
    }
    
    /** Set a rectangel for zooming.
    * @param zoom the Rectangnle for zooming.
    */
    public void setZoom(Rectangle2D.Float zoom) {
        this.zoom = zoom;
    }
    
    /** All painting activities are here. */
    public void paintComponent(Graphics g) {

        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d2 = (Dimension2D) getSize();
        float w = (float) d2.getWidth();
        float h = (float) d2.getHeight();
        
        try {
            rel = (h - 2 * yPadding) / range.getExtendF();
            offset = range.getEndF();
        }
    catch (UnknownPositionException e) { System.out.println(e); }

        // Clear background.
        g2.setColor(bColor);
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

        // Drawing points to show locus distribution.
        if (points.size() != 0) {
            rightP = drawPoints(g2, h);
            if (mag != null) mag.setVerTicalPosition(rightP);
            //rightP = 10.0f;
        }

        float x = rightP + 5.0f;
        // Drawing line to represent the cartoon.
        g2.setColor(vColor);
        //g2.fill(new Ellipse2D.Float(x - 3.0f, yPadding - 3.0f, 6.0f, 6.0f));
        //g2.fill(new Ellipse2D.Float(x - 3.0f, h - yPadding - 3.0f, 6.0f, 6.0f));
        g2.draw(new Line2D.Float(x, yPadding, x, h - yPadding));
        add(topKnob);
        topKnob.setBounds((int)(x - 4.0f), (int)(yPadding - 8.0f), 8, 8);
        add(bottomKnob);
        bottomKnob.setBounds((int)(x - 4.0f), (int)(h - yPadding), 8, 8);

        // Drawing marks for all loci in range on the above line.
        g2.setColor(hColor);
        //if ((loci != null) && (loci.length != 0)){
        if (sTotal != null) {
            for (int i = 0; i < sTotal.length; i++) {
                try {
                    float pos = sTotal[i].getPosition();
                    float y = (offset - pos) * rel + yPadding;
                    g2.draw(new Line2D.Float(x - 5.0f, y, x + 5.0f, y));
                }
            catch (UnknownPositionException e) { System.err.println(e); }
            }
        }
        // Setting the positions of locusUIs.
        x += 5;

        // Layout the locusUI
        layout.drawNames(x, h, offset, rel, g2);

        if (zoom != null) {
            g2.setPaint(Color.blue);
            g2.draw(zoom);
        }
    }

    /** A helper for drawing MapPoints. */
    private float drawPoints(Graphics2D g2, float h) {
        int size = points.size();
        float x = xPadding;
        g2.setColor(pColor);

        for (int i = size - 1; i >= 0; i --) {
            Vector v = (Vector) points.elementAt(i);
            int size1 = v.size();
            for (int j = 0; j < size1; j++) {
                try {
                    MapPoint p = (MapPoint) v.elementAt(j);
                    float pos = p.getPosition();
                    float y = (offset - pos) * rel + yPadding;
                    if ((y < yPadding) || (y > (h - yPadding))) continue;
                    g2.draw(new Ellipse2D.Float(x - 1.0f, y - 1.0f, 2.0f, 2.0f));
                }
            catch (UnknownPositionException e) { System.err.println(e); }
            }
            x = x + 3.0f;
        }

        return x;
    }

    // MouseListener methods
    /** Do nothing. */
    public void mouseEntered(MouseEvent e) {
    }

    /** Showing popup menu. */
    public void mousePressed(MouseEvent e) {
        if (e.isPopupTrigger()) {
            doPopup(e);
        }
    }

    /** Showing popup menu. */
    public void mouseReleased(MouseEvent e) {
        if (e.isPopupTrigger()) doPopup(e);
    }

    /** Do nothing. */
    public void mouseClicked(MouseEvent e) {
    }

    /** Do nothing. */
    public void mouseExited(MouseEvent e) {
    }

    // show popup.
    private void doPopup(MouseEvent e) {
        pop2.show(this, e.getX(), e.getY());
    }

    /** Search loci with names containg pattern.
    * @param pattern the searching key.
    */
    public void doSearch(String pattern) {
        if (control != null) control.doSearch(pattern);
    }

    // MouseMotion method to draw zoom rectange
    /** Do nothing. */
    public void mouseDragged(MouseEvent e) {
    }

    /** Do nothing. */
    public void mouseMoved(MouseEvent e) {
    }

    /** Actual zooming method. zoom based on the rectangle drawed by the mouse. */
    public void doZooming() {
        if (zoom != null) {
            inY = (float) zoom.getMinY();
            outY = (float) zoom.getMaxY();
            float start = offset - (outY - yPadding) / rel;
            float end = offset - (inY - yPadding) / rel;
            //System.out.println(start + "    " + end);
            MapRange r1 = new MapRange(start, end);
            //System.out.println("doZooming in LocusDisplay: " + start + "   " + end);
            if (control != null) control.zoom(r1);
            zoom = null;
        }
    }

    // main method for testing.
    public static void main(String[] args) {
        JFrame f = new JFrame("Locus Display");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        MapRange r = new MapRange(-30.0f, 30.0f);
        AbstractMapPoint[] p = new AbstractMapPoint[20];
        for (int i = 0; i < 6; i++)
        p[i] = new AbstractMapPoint("Point" + i, 5.0f);
        for (int i = 6; i < 12; i++)
        p[i] = new AbstractMapPoint("Point" + i, 13.0f);
        for (int i = 12; i < 18; i++)
        p[i] = new AbstractMapPoint("Point" + i, -3.9f);
        p[18] = new AbstractMapPoint("Point" + 18, -4.5f);
        p[19] = new AbstractMapPoint("Point" + 19, 8.9f);
        DisplayModel m = new DisplayModel();
        m.setData(p);
        LocusDisplay display = new LocusDisplay(r);
        display.setModel(m);
        //display.setShowNumber(20);
        f.getContentPane().add(display, "Center");
        f.setSize(300, 500);
        f.setVisible(true);
    }
}