/*
 * MapRange.java
 *
 * Created on July 11, 2000, 4:34 PM
 */

package jadex.swing.map;

/**
 * This is a utility class for storing the positions of the MapPoint, and processing
 * the range calculations.
 * @author  wgm
 * @version 1.0
 */
public class MapRange extends Object {

    private long start = Long.MAX_VALUE; // The start position
    private long end = Long.MAX_VALUE; // The end position 
    private float startF = Float.POSITIVE_INFINITY; // start position in float.
    private float endF = Float.POSITIVE_INFINITY; // end position in float.

    private long extend; // The extend of this range.
    /** Default constructor */
    public MapRange() {
    }

    /** MapRange with integer numbers will be used in SequenceMap and PhysicalMap.
    * @param start the start position 
    * @param end the end position.
    */
    public MapRange(long start, long end) {
        setStart(start);
        setEnd(end);
    }

    /** MapRange with float point numbers will be used in GeneticMap.
    * @param start the start position in float.
    * @param end the end position in float.
    */
    public MapRange(float startF, float endF) {
        setStartF(startF);
        setEndF(endF);
    }

    /** Construct a maprange with String arguments.
    * @param left the start position in string.
    * @param right the end position in string.
    */
    public MapRange(String left, String right) {
        try { // If this is OK, the postion is integers
            int start = Integer.parseInt(left);
            int end = Integer.parseInt(right);
            setStart(start);
            setEnd(end);
        }
        catch(NumberFormatException e) { // Otherwise, the numbers are floats
            try {
                float startF = Float.parseFloat(left);
                float endF = Float.parseFloat(right);
                setStartF(startF);
                setEndF(endF);
            }
            catch(NumberFormatException e1) { // Something is wrong
                System.err.println("MapRange.init(): " + e1);
            }
        }
    }

    /** Set the start value.
    * @param s the start value.
    */
    public void setStart(long s) {
        this.start = s;
        this.startF = (float) s;
    }

    /** Set the end value.
    * @param e the end value.
    */
    public void setEnd(long e) {
        this.end = e;
        this.endF = (float) e;
    }

    /** Get the start value.
    * @return the start value.
    * @exception throw an UnknownPositionException if the position is unknown.
    */
    public long getStart() throws UnknownPositionException {
        if (start == Long.MAX_VALUE) throw new UnknownPositionException();
        return this.start;
    }

    /** Get the end value.
    * @return the end value.
    * @exception throw an UnknownPositionException is the position is unknown.
    */
    public long getEnd() throws UnknownPositionException {
        if (end == Long.MAX_VALUE) throw new UnknownPositionException();
        return this.end;
    }

    /** Get the total length of this MapRange.
    * @return the total length of this MapRange.
    * @exception throw an UnknownPositionException is the value is not initialized.
    */
    public long getExtend() throws UnknownPositionException {
        if ((start == Long.MAX_VALUE) || (end == Long.MAX_VALUE)) throw new UnknownPositionException();
        long diff = end - start;
        if (diff >= 0) return diff + 1;
        else return diff - 1;
    }

    /** Get the total length in float.
    * @return the total length of this MapRange in float.
    * @exception throw an UnknownPositionException if the value is not initialized.
    */
    /** How to include the two ends? This is a question? */
    public float getExtendF() throws UnknownPositionException {
        if ((start == Float.POSITIVE_INFINITY) || (end == Float.POSITIVE_INFINITY))
        throw new UnknownPositionException();
        else return endF - startF;
    }

    /** An handy print method.
    */
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("Start: " + start);
        buffer.append("    ");
        buffer.append("End: " + end);
        //buffer.append(".");
        return buffer.toString();
    }

    /** For float values.
    * @deprecated use {@link toString() toString}.
    */
    public String toStringF() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("Start: " + startF);
        buffer.append(" ");
        buffer.append("End: " + endF);
        return buffer.toString();
    }

    /** Compare two mapranges.
    * @return true if both start and end position are the same.
    */
    public boolean equals(MapRange rg) {
        boolean rtn = false;
        try {
            if ((getStart() == rg.getStart()) && (getEnd() == rg.getEnd())) rtn = true;
        }
    catch (UnknownPositionException e) { System.out.println(e); }
        return rtn;
    }

    /** Compare two MapRanges in float values.
    * @deprecated use {@link equals(MapRange) equals(MapRange rg)}.
    */
    public boolean equalsF(MapRange rg) {
        boolean rtn = false;
        try {
            if ((getStartF() == rg.getStartF()) && (getEndF() == rg.getEndF())) rtn = true;
        }
    catch (UnknownPositionException e) { System.out.println(e); }
        return rtn;
    }

    /** Get the start value in float.
    * @return the start value in float.
    */
    public float getStartF() throws UnknownPositionException {
        if (startF == Float.POSITIVE_INFINITY) throw new UnknownPositionException();
        else return startF;
    }

    /** Get the end value in float.
    * @return the end value in float.
    */
    public float getEndF() throws UnknownPositionException {
        if (endF == Float.POSITIVE_INFINITY) throw new UnknownPositionException();
        else return endF;
    }

    /** Set the start value in float.
     */
    public void setStartF(float start) {
        this.startF = start;
        this.start = (long) start;
    }

    /** Set the end value in float. */
    public void setEndF(float end) {
        this.endF = end;
        this.end = (long) end;
    }

    /** Check if two mapranges intersect.
     * @param r the compared maprange
     * @return true if intersecting
     */
    public boolean intersect(MapRange r) {
        boolean rtn = true;
        try {
            float start1 = r.getStart();
            float end1 = r.getEnd();
            if ((end <= start1) || (start >= end1))  rtn = false;
        }
        catch(UnknownPositionException e) {
            System.err.println("MapRange.intersect: " + e);
        }

        return rtn;
    }

    /** Zoom-in in the current range
     * @param the strength of the zooming
     * @return a new maprange
     */
    public MapRange zoomIn(int step) {
        long extend = (end - start) / step;
        return new MapRange((start + extend), (end - extend));
    }

    /** Zoom-out the current range
     * @param the strength of the zooming
     * @return this object changed
     */
    public MapRange zoomOut(int step) {
        long extend = (end - start) / step;
        return new MapRange(start - extend, end + extend);
    }

    /** Move the current range to the up direction.
    * @param step the magnitude of the moving.
     */
    public MapRange moveUp(int step) {
        long extend = (end - start) / step;
        return new MapRange(start + extend, end + extend);
    }

    /** Move the current range to the lower direction.
    * @param step the magnitude of the moving.
    */
    public MapRange moveDown(int step) {
        long extend = (end - start) / step;
        return new MapRange(start - extend, end - extend);
    }

    /** Check if this maprange contains a specific value.
     * This is an inclusive comparison.
     * @param value the compared value.
     * @return true if this MapRange contains this value.
     */
    public boolean contain(long value) {
        // First check it when in positive direction
        if (start < end) {
            if ((value >= start) && (value <= end)) return true;
            else return false;
        }
        // Then check when in a negative direction
        else {
            if ((value >= end) && (value <=  start)) return true;
            else return false;
        }
    }
    
    /** Add two maprange together and return a changed this maprange 
     * @param range1 the maprange to be added
     * @return a new maprange cover both this and range1
    */
    
    public MapRange combine(MapRange range1) {
        // return a clone of this object
        if (range1 == null) return new MapRange(start, end);
        
        long start1 = this.start;
        long end1 = this.end;
        if (start1 > end1) {
            start1 = this.end;
            end1 = this.start;
        }
        
        long start2 = 0;
        long end2 = 0;
        try {
            start2 = range1.getStart();
            end2 = range1.getEnd();
            if (start2 > end2) {
                start2 = range1.getEnd();
                end2 = range1.getStart();
            }
        }
        catch(UnknownPositionException e) {
            System.err.println("MapRange.combine(): " + e);
        }
        
        long s = (start1 < start2) ? start1 : start2;
        long e = (end1 > end2) ? end1 : end2;
        return new MapRange(s, e);
    }
    
    /** Get the left end of this maprange */
    public long getLeftEnd() {
        if (start < end) return start;
        else return end;
    }
    
    /** Get the right end of this maprange */
    public long getRightEnd() {
        if (start < end) return end;
        else return start;
    }
}