/*
 * CloneDisplay.java: This is the class used to show clones in the chromosome. Many methods of
 * this class are from LocusDisplay. Therefore, LocusDisplay and this class should be refactored.
 *
 * Created on October 6, 2000, 3:45 PM
 */

package jadex.swing.sequence;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;

import jadex.swing.map.*;
import jadex.swing.util.*;
/**
 * This is the class used to show clones in the chromosome. Many methods of this class are from LocusDisplay.
 * Therefore, LocusDisplay and this class should be refactored.
 * @author  Guanming Wu
 * @version 1.0
 */
public class CloneDisplay extends JPanel implements MapPointDisplay, Zoomable, MouseListener {

    private CloneControl control = null; // The CloneControl to control this display
    private java.util.List clones = null; // Total clones in this display
    private java.util.List assClones = null; // The distributed clone list: List of List
    private LocusUI[] names = null; // Showing clone names in the current range
    private MapRange range = null; // The current maprange in this display

    private boolean cShown = true; // State if the connected lines should be shown
    private String mouseInName = null; // The clone name where mouse is in

    // variables for controlling the appearance of the image
    private final float xPadding = 10.0f;
    private final float yPadding = 10.0f;
    private final float CLONE_WIDTH = 10.0f; // The widht of the line for clones
    private final int TOTAL_COLUMN = 4; // Total column numer for clones on the left of Vertical Line
    private float rel; // Ratio between absolute coordinate and maprange
    private float offset; // Offset to convert these two coordinate systems
    private float verticalP; // The x pos of the vertical line for chromosome
    private Rectangle2D.Float zoom = null; // For zooming purpose

    // Tow knobs for controling the movement of the maprange on the vertival bar
    private Knob upKnob = null;
    private Knob downKnob = null;
    private DisplayPopup pop = null; // The popup menu for controlling this display

    // Locus layout for controling the names: The class names should be changed
    private LocusLayout layout = null; // default layout
    private LocusLayout columnLayout = null;
    private LocusLayout smartLayout = null;

    // A magnifier for zooming
    private Magnifier mag = null;

    /** Default constructor */
    public CloneDisplay() {
        initialize();
    }

    /** Second constructor: CloneControl as the argument */
    public CloneDisplay(CloneControl control) {
        initialize();
        setCloneControl(control);
    }

    /** Third constructor: DisplayModel as the argument */
    public CloneDisplay(DisplayModel model) {
        initialize();
        setClones(Arrays.asList(model.getData()));
    }

    // All initializing activities are here
    private void initialize() {
        super.setLayout(null);
        super.addMouseListener(this);
        setKnobs();
        mag = new Magnifier(this);
        columnLayout = new LocusLayout(yPadding, DisplayControl.NUMBER_PER_COLUMN, cShown, Color.lightGray);
        smartLayout = new SmartLocusLayout(yPadding, DisplayControl.NUMBER_PER_COLUMN, cShown, Color.lightGray);
        layout = columnLayout;
    }

    /** Get the PopupMenu used in this CloneDisplay.
    */
    public DisplayPopup getPopup() {
        return pop;
    }

    // Set up the knobs
    private void setKnobs() {
        upKnob = new UpKnob();
        downKnob = new DownKnob();
        add(upKnob);
        add(downKnob);
        upKnob.setBounds(0, 0, 10, 10);
        downKnob.setBounds(0, 0, 10, 10);
        if (control != null) {
            control.setKnobs(upKnob, downKnob);
            upKnob.setDisplayControl(control);
            downKnob.setDisplayControl(control);
        }
    }

    /** Set the control of the display.
    @param ctl the CloneControl used for this CloneDisplay.
    */
    public void setCloneControl(CloneControl ctl) {
        this.control = ctl;
        pop = new DisplayPopup(this, control, "clone");
        setClones(control.getClones());
        assClones = control.getAssignedClones();
        verticalP = xPadding * (assClones.size() + 1);
        mag.setVerTicalPosition(verticalP);
        control.setKnobs(upKnob, downKnob);
        upKnob.setDisplayControl(control);
        downKnob.setDisplayControl(control);
    }

    /** Set the clones.
    * @param list the clones to be displayed.
    */
    public void setClones(java.util.List list) {
        clones = list;
    }

    /** Set the current maprange.
    * @param r the current MapRange to be displayed.
    */
    public void setRange(MapRange r) {
        this.range = r;
        repaint();
    }

    /** Set the nameLayout.
    * @param l the name layout for layouting names of clones in the display.
    */
    public void setNameLayout(LocusLayout l) {
        this.layout = l;
        layout.setLocusUI(names);
        repaint();
    }

    /** Set the name layout by layout name. LocusLayout can also be set by their names.
    * @param name the name of the layout to be used in this CloneDisplay.
    */
    public void setNameLayout(String name) {
        if (name.equalsIgnoreCase("columnLayout")) {
            setNameLayout(columnLayout);
        }
        else if (name.equalsIgnoreCase("smartLayout")) setNameLayout(smartLayout);
    }

    /** Set if the connecting lines should be displayed.
    * @param b showing the connecting lines if true.
    */
    public void setConnectShown(boolean b) {
        cShown = b;
        //layout.setConnect(b);
        columnLayout.setConnect(b);
        smartLayout.setConnect(b);
        repaint();
    }

    /** Get the status of the connecting lines.
    * @return true if the connecting lines are shown.
    */
    public boolean isConnectShown() {
        return cShown;
    }

    /** Names will be shown in this range. Repaint() should be called explicitily.
    * @param the names of clones will be displayed.
    */
    public void setShowNames(LocusUI[] l) {
        names = l;
        this.removeAll();
        // Add back the knobs
        add(upKnob);
        add(downKnob);

        if ((names != null) && (names.length > 0)) {

            for (int i = 0; i < names.length; i++) {
                add(names[i]);
                names[i].setBounds(0, 0, 40, 10); //Have to set the original boounds
                names[i].addMouseListener(new LineMouse(names[i], this));
            }
        }
        layout.setLocusUI(names);
    }

    /** The most important method in this class. All displaying activities are here.
    */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d2 = (Dimension2D) getSize();
        float w = (float) d2.getWidth();
        float h = (float) d2.getHeight();
        try {
            offset = range.getEndF();
            rel = (h - 2 * yPadding) / range.getExtendF();
        }
        catch(UnknownPositionException e) {
            System.err.println("CloneDisplay.paintComponent(): " + e);
        }

        g2.setPaint(Color.white);
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));

        drawClones(g2, h);
        drawChromosome(g2, h);

        layout.drawNames(verticalP + xPadding / 2.0f, h, offset, rel, g2);
        if (zoom != null) {
            g2.setPaint(Color.blue);
            g2.draw(zoom);
        }
    }

    /** Draw all clones as a heavy vertical lines on the left of the
     *   vertical chromosome line
     */
    private void drawClones(Graphics2D g2, float h) {
        if (assClones != null) {
            Clone clone = null;
            float x = 0.0f;
            for (Iterator it = assClones.iterator(); it.hasNext();) {
                java.util.List clones = (java.util.List) it.next();
                x += xPadding;
                for (Iterator iter = clones.iterator(); iter.hasNext(); ) {
                    clone = (Clone) iter.next();
                    drawClone(clone, g2, x, h);
                }
            }
        }
    }

    /** A helper method to draw individual clone */
    private void drawClone(Clone clone, Graphics2D g2, float x, float h) {
        MapRange range = clone.getMapRange();
        try {
            float s = range.getStartF();
            float e = range.getEndF();
            float p = clone.getPosition();
            float start = (offset - s) * rel + yPadding;
            float end  = (offset - e) * rel + yPadding;
            float pos = (offset - p) * rel + yPadding;
            if ((pos < yPadding) || (pos > (h - yPadding))) return ;
            if (clone.getName().equalsIgnoreCase(mouseInName)) {
                g2.setPaint(Color.red);
                g2.draw(new Line2D.Float(x, start, x, end)); // Vertical line
                g2.setPaint(Color.red);
                g2.draw(new Line2D.Float(x, pos, verticalP + CLONE_WIDTH / 2.0f, pos));
            }
            else {
                g2.setPaint(Color.orange);
                g2.draw(new Line2D.Float(x, start, x, end)); // Vertical line
                g2.setPaint(Color.black);
                g2.draw(new Line2D.Float(verticalP - CLONE_WIDTH / 2.0f, pos, verticalP + CLONE_WIDTH / 2.0f, pos));
            }
        }
        catch(UnknownPositionException e) {
            System.err.println("CloneDisplay.drawClone(): " + e);
        }
    }

    /** Draw vertical lines and horizotal lines to represent the chromosome
     *  and clones on the chromosome
     */
    private void drawChromosome(Graphics2D g2, float h) {
        float x = verticalP;
        g2.setColor(Color.lightGray);
        g2.draw(new Line2D.Float(x, yPadding, x, h - yPadding));
        upKnob.setBounds((int)(x - 4.0f), (int)(yPadding - 8.0f), 8, 8);
        downKnob.setBounds((int)(x - 4.0f), (int)(h - yPadding), 8, 8);
    }

    /** Get the position of the chromosome line.
    * @return the position of the chromosome line.
    */
    public float getVerticalPosition() {
        return verticalP;
    }
    /** Zoom range should be passed from magnifier to here.
     * After zooming,  zoom should be clear up, i.e., zoom = null.
    * @param zoom the rectangle to be zoomed.
    */
    public void setZoom(Rectangle2D.Float zoom) {
        this.zoom = zoom;
    }

    /** Do zooming.
    */
    public void doZooming() {
        if (zoom != null) {
            float inY = (float) zoom.getMinY();
            float outY = (float) zoom.getMaxY();
            float start = offset - (outY - yPadding) / rel;
            float end = offset - (inY - yPadding) / rel;
            MapRange r1 = new MapRange(start, end);
            if (control != null) control.zoom(r1);
            else setRange(r1);
            zoom = null;
        }
    }

    /** Show popup menu. For windows. */
    public void mouseReleased(final java.awt.event.MouseEvent p1) {
        if (p1.isPopupTrigger())  doPopup(p1);
    }
    /** Do  nothing. */
    public void mouseEntered(final java.awt.event.MouseEvent p1) {
    }
    /** Do nothing. */
    public void mouseClicked(final java.awt.event.MouseEvent p1) {
    }
    /** Show popup menu. For linux. */
    public void mousePressed(final java.awt.event.MouseEvent p1) {
        if (p1.isPopupTrigger()) doPopup(p1);
    }
    public void mouseExited(final java.awt.event.MouseEvent p1) {
    }

    // show popup.
    private void doPopup(MouseEvent e) {
        if (pop != null) {
            pop.show(this, e.getX(), e.getY());
        }
    }

    // A Tester method
    public static void main(String[] args) {
        CloneFetcher fetcher = new FileCloneFetcher("sequence_map_I");
        fetcher.fetchClone();
        CloneDisplay display = new CloneDisplay();
        //display.setRange(new MapRange(0, 3000));
        CloneControl control = new CloneControl(display);
        MapRange r = new MapRange(0, 7867);
        display.setRange(r);
        control.setCloneFetcher(fetcher);
        control.setMapRange(r);
        display.setCloneControl(control);

        // Setup the layout
        JFrame f = new JFrame("Clone Display: Chromosome I");
        f.addWindowListener(new WindowAdapter() {
        public void windowClosing(WindowEvent e) { System.exit(0); }
        });
        f.getContentPane().add(new JScrollPane(display), "Center");
        display.setSize(400, 600);
        f.setSize(400, 600);
        f.setVisible(true);
    }
    
    /** Get the name layout used in this CloneDisplay. 
    * @return the name layout currently used in this clonedisplay.
    */
    public LocusLayout getNameLayout() {
        return this.layout;
    }
    /** Set the name in which the mouse is in.
    * @param name the name in which the mouse is in.
    */
    public void setMouseInName(String name) {
        this.mouseInName = name;
        repaint();
    }
}