/*
 * CloneFetcher.java: This class is used to get clone data from outside
 *
 * Created on September 26, 2000, 11:00 AM
 */

package jadex.swing.sequence;

import java.io.*;
import java.util.*;
import jadex.swing.map.*;

/**
 * This class is used to get clone data from a datasource, such as AceDB or local files.
 * @author  wgm
 * @version 1.0
 */
public  abstract class CloneFetcher extends Object {

    private String name = null; // The sequence_map name of this clone in
    private MapRange map = null; // The total range of the chromosome should be got
    protected static final String link = "_"; // The link character used in string reading
    private Map clones = null; // All clones fetched
    private Map ctgs = null; // The contigs of the sequence_map

    // A small class for holding the information of line from the string
    private class LineInfor {
        String str1;
        String str2;
        MapRange range;

    LineInfor() {}
    };

    /** The main method to be called by Model or other class to get the data.
     * This method should be called explicitly. This is a template method based on
    * the book, Design Patterns.
     */
    public void fetchClone() {
        String str = getCloneInfor();
        getClone(str);
    }

    /** Get the clone information for the Sequence_map into a String.
    * @return a String containing all clone information.
    */
    public abstract String getCloneInfor();

    /** Parse clone information from String.
    * @param str the String containing all clone information.
    */
    protected void getClone(String str) {
        try {
            BufferedReader reader = new BufferedReader(new StringReader(str));
            clones = new HashMap();
            String line = null;
            name = reader.readLine().trim(); // The first line is the map name
            while ((line = reader.readLine()) != null) {

                // Starting from the second line is information on contigs if any
                if (line.trim().equalsIgnoreCase(name + link + "contig")) readContig(reader);
                else if (line.trim().equalsIgnoreCase(name + link + "contig" + link + "clone")) readContigClone(reader, clones);
                else if (line.trim().equalsIgnoreCase(name + link + "clone")) readClone(reader, clones);
                else if (line.trim().equalsIgnoreCase(name + link + "infor")) readSequenceInfor(reader);
            }
        }
        catch (IOException e) {
            System.err.println("CloneFetcher.getClone(): " + e);
        }
    }

    // A helper to get the information of contigs
    private void readContig(BufferedReader r) {
        String line = null;
        ctgs = new HashMap();

        try {
            while (true) {
                line = r.readLine().trim(); // This line is the name of the contig
                if ((line == null) || (line.length() == 0)) break;
                else {
                    Contig ctg = new Contig();
                    ctg.setName(line);
                    line = r.readLine(); // Remove the format line
                    if (line.startsWith("Format")) line = r.readLine();
                    if ((line.indexOf(name)) == -1) {// This line contains positions of geneticMap
                        //System.out.println("1: " + line);
                        LineInfor infor = parseLine(line);
                        ctg.setExtendF(infor.range);
                        line = r.readLine();
                    }
                    //line = r.readLine(); // Position on the Sequence_map
                    //System.out.println("2: " + line);
                    LineInfor infor = parseLine(line);
                    ctg.setExtend(infor.range);
                    line = r.readLine();
                    if (line.startsWith("//")) line = r.readLine(); // Remove comment line
                    if (line.startsWith("Format")) line = r.readLine();
                    //r.readLine(); // Remove the Format line
                    //line = r.readLine(); // PMap
                    infor = parseLine(line);
                    ctg.setPMap(infor.range);
                    ctgs.put(ctg.getName(), ctg);
                    r.readLine(); // Remove comment line
                }
            }
        }
        catch(IOException e) {
            System.err.println("CloneFetcher.readContig(): " + e);
        }
    }

    // Read the clones from contigs
    private void readContigClone(BufferedReader reader, Map clones) {
        String line = null;
        try {
            while (true) {
                line = reader.readLine();
                if ((line == null) || (line.length() == 0)) break;
                if (line.indexOf("Format") > -1) continue; // Get rid of format lines
                if (line.startsWith("//")) continue; // Get rid of comment line
                else {
                    LineInfor infor = parseLine(line);
                    Clone clone = new Clone(infor.str1, infor.range, (Contig) ctgs.get(infor.str2));
                    clones.put(clone.getName(), clone);
                }
            }
        }
        catch(IOException e) {
            System.err.println("CloneFetcher.readContigClone(): " + e);
        }
    }

    // Read the clones from sequence_map
    private void readClone(BufferedReader reader, Map clones) {
        String line = null;
        try {
            while (true) {
                line = reader.readLine().trim();
                if ((line == null) || (line.length() == 0)) break;
                if (line.startsWith("Format")) continue;
                if (line.startsWith("//")) continue;
                else {
                    LineInfor infor = parseLine(line);
                    if (clones.containsKey(infor.str1)) continue; // This clone has already in the map
                    Clone clone = new Clone(infor.str1, infor.range); // No contig information in this clone
                    clones.put(clone.getName(), clone);
                }
            }
        }
        catch (IOException e) {
            System.err.println("CloneFetcher.readClone(): " + e);
        }
    }

    // get the extent of sequence_map
    private void readSequenceInfor(BufferedReader reader) {
        try {
            reader.readLine(); // Get rid of Format line
            String line = reader.readLine(); // Only one line information
            LineInfor infor = parseLine(line);
            map = infor.range;
            reader.readLine(); // get rid of comment line
        }
        catch (IOException e) {
            System.err.println("CloneFetcher.readSequenceInfor(): " + e);
        }
    }

    // A helper method to parse the string line
    private LineInfor parseLine(String line) {
        //System.out.println("CloneFetcher.parseLine: " + line);
        LineInfor rtn = new LineInfor();
        // Quotation marks should be reomved
        StringTokenizer tokenizer = new StringTokenizer(line);
        if (tokenizer.countTokens() == 3) { // First is string, second and third are numbers
            rtn.str1 = tokenizer.nextToken();
        }
        else if (tokenizer.countTokens() == 4) { // First and second should be strings, third and fourth are numbers
            rtn.str1 = tokenizer.nextToken();
            rtn.str2 = tokenizer.nextToken();
        }

        String left = tokenizer.nextToken(); // the left end
        String right = tokenizer.nextToken(); // The right end
        MapRange r = new MapRange(left, right);
        rtn.range = r;
        return rtn;
    }

    /** Get the clones. Please note that the returned value is a Map.
    * @return the clones to be displayed.
    */
public Map getClone() { return clones; }

    /** Get the name of the Sequence_map to be displayed.
    * @return the name of the Sequence_map to be displayed.
    */
public String getMapName() { return this.name; }

    /** Get the contigs in this Sequence_maps. 
    * @return the contigs. The returned is a map.
    */
public Map getContig() { return ctgs; }

    /** Get the total MapRange for the Sequence_map.
    * @return the MapRange of the Sequence_map.
    */
public MapRange getMapScale() { return map; }

}