/*
 * GeneRenderer.java: A JComponent renderer a gene element will be drawn on
 *
 * Created on December 1, 2000, 1:55 PM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

import org.jdom.*;

import jadex.swing.map.MapRange;
/**
 * A customized JComponent for rendering gene structures in SequenceView.
 * @author  wgm
 * @version 1.0
 */
public class GeneRenderer extends JComponent {

    // Color mode
    private Color background = null; // Want to get background from SequenceDisplay
    private Color foreground = Color.yellow;
    private java.util.List exons = null; // exons is a list of exons
    
    // Dimension mode
    private float xPadding = 1.0f;
    private float yPadding = 1.0f;
    private long totalBp; // The total bp length of the gene
    
    /** Default constructor */
    public GeneRenderer() {
        setBorder(BorderFactory.createEtchedBorder());
        setLayout(null);
    }
    
    /** Get the gene renderer for a gene element. The calling method shold call
     * this component's validate() method explicitly.
     * @param gene an gene element
     * @display the sequence display this renderer will be painted
     * @length the length of the gene, i.e., the parent sequence length
     * @return the gene renderer
     */
    public JComponent getGeneRenderer(Element gene, JComponent display, long length) {
        background = display.getBackground();
        this.totalBp = length;
        constructExons(gene);
        return this;
    }
       
    /** A helper to extract exons information from gene element */
    private void constructExons(Element gene) {
        Namespace ns = gene.getNamespace();
        Element exonsElement = gene.getChild("Exons", ns);
        exons = exonsElement.getChildren("Exon", ns);
    }
    
    /** All draw activites are here */
    public void paintComponent(Graphics g) {
        // super.paintComponent(g); // It is not necessary to call this method here.
        Graphics2D g2 = (Graphics2D) g;
        Dimension2D d2 = (Dimension2D) getSize();
        Insets insets = getInsets();
        float w = (float) d2.getWidth();
        float h = (float) d2.getHeight();
        // The switch factor: x / bp
        float rel = (w - 2 * xPadding - insets.left - insets.right) / totalBp; 
        float startX = xPadding + insets.left;
        if (totalBp < 0) { // Negative direction
            startX = w - xPadding - insets.right;
        }
        // Clear up the background
        g2.setPaint(background);
        g2.fill(new Rectangle2D.Float(0.0f, 0.0f, w, h));
        // Draw exons 
        Element exon = null;
        g2.setPaint(foreground);
        float exonX1 = -1.0f;
        float pos[] = new float[]{-1.0f, -1.0f};
        int size = exons.size();
        for (int i = 0; i < size; i ++) {
            exon = (Element) exons.get(i);
            exonX1 = pos[1]; 
            pos = paintExon(exon, g2, rel, startX, h);
            if (i == 0) continue; // No needing to paint intron
            paintIntron(g2, exonX1, pos[0], h);
        }
    }
    
    /** Paint an exon */
    private float[] paintExon(Element exon, Graphics2D g2, float rel, float startX, float h) {
        String start = exon.getAttributeValue("start");
        String end = exon.getAttributeValue("end");
        int s = Integer.parseInt(start);
        int e = Integer.parseInt(end);
        float x1 = startX + s * rel;
        float x2 = startX + e * rel;
        if (rel > 0) g2.fill(new Rectangle2D.Float(x1, h / 3.0f, (x2 - x1), h / 3.0f)); // Positive direction
        else g2.fill(new Rectangle2D.Float(x2, h / 3.0f, (x1 - x2), h / 3.0f)); // Negative direction
        return new float[]{x1, x2};
    }   
    
    /** Paint an intron */
    private void paintIntron(Graphics2D g2, float exonX1, float exonX2, float h) {
        float mid = (exonX1 + exonX2) / 2.0f;
        g2.draw(new Line2D.Float(exonX1, h / 2.0f, mid, h / 6.0f));
        g2.draw(new Line2D.Float(exonX2, h / 2.0f, mid, h / 6.0f));
    }
}