/*
 * SequenceActions.java: This is the central repository to handle all actions 
 * in the whole Sequence Display. This class is a singleton.
 *
 * Created on December 6, 2000, 3:18 PM
 */

package jadex.swing.sequence;

import java.awt.event.*;
import java.awt.*;
import java.beans.*;
import java.util.*;

import javax.swing.*;
import jadex.swing.map.MapRange;

/**
 * This is the central repository to handle all actions in the whole Sequence Display. 
 * This class is a singleton.
 * @author  wgm
 * @version 1.0
 */
public class SequenceActions extends Object {

    private Action zoom = null; // Customized zoom action
    private Action zoomIn = null; // Zoom In function
    private Action zoomOut = null; // Zoom out function
    private Action reset = null; // Reset to the whole range function
    private Action moveLeft = null; // Move the range to the left
    private Action moveRight = null; // Move the range to the right
    private Action search = null; // Search a specify sequence action
    private Action help = null; // help action
    private Action exit = null; // Exit action
    
    private Map actions = new HashMap();
    
    //private SequenceViewInterface view = null;
    private SequenceDisplay display = null;
    
    private boolean inApplet = false; // Default in an application
    
    // These two variables are used to control the icon size on the toolbar
    private final int BIG_ICON = 24;
    private final int SMALL_ICON = 16; // Default size
    private int iconSize = SMALL_ICON;
    
    private static SequenceActions instance = null; // An singleton reference
    
    /** The method to get an instance of this class. */
    public static SequenceActions getInstance() {
        if (instance == null) {
            instance = new SequenceActions();
        }
        return instance;
    }
    
    /** Another method to get an instance of this class.
    * @param inApplet true if the SequenceView is in an applet.
    */
    public static SequenceActions getInstance(boolean inApplet) {
        if (instance == null) {
            instance = new SequenceActions(inApplet);
        }
        return instance;
    }
    
    /** Creates new SequenceActions. */
    private SequenceActions() {
        initialize();
    }

    /** Another private constructor used in an applet. */
    private SequenceActions(boolean inApplet) {
        this.inApplet = inApplet;
        initialize();
    }
    
    /** Initialization function */
    private void initialize() {
        makeZoomAction();
        makeZoomInAction();
        makeZoomOutAction();
        makeResetAction();
        makeMoveLeftAction();
        makeMoveRightAction();
        makeSearchAction();
        makeHelpAction();
        makeExitAction();
    }
    
    /** Query if in applet.
    * @return true if in an applet.
    */
    public boolean isInApplet() {
        return inApplet;
    }
    
    /** Make a default zoom action */
    private void makeZoomAction() {
        ImageIcon icon = getIcon("Zoom");
        zoom = new AbstractAction("Specify a Range", icon) {
            public void actionPerformed(ActionEvent e) {
                RangeSpecifyDialog dialog = new RangeSpecifyDialog(new JFrame(), true);
                dialog.addPropertyChangeListener(new PropertyChangeHandler());
                dialog.show();
            }
        };
        actions.put("zoom", zoom);
    }
    
    /** Make a default zoomInAction */
    private void makeZoomInAction() {
        ImageIcon icon = getIcon("ZoomIn");
        zoomIn = new AbstractAction("Zoom In", icon) {
            public void actionPerformed(ActionEvent e) {
                if (display != null) {
                    display.getLayoutModel().zoomIn();
                }
            }
        };
        actions.put("zoomIn", zoomIn);
    }
    
    /** Make a default zoomOutAction */
    private void makeZoomOutAction() {
        ImageIcon icon = getIcon("ZoomOut");
        zoomOut = new AbstractAction("Zoom Out", icon) {
            public void actionPerformed(ActionEvent e) {
                if (display != null) {
                    display.getLayoutModel().zoomOut();
                }
            }
        };
        actions.put("zoomOut", zoomOut);
    }
    
    /** Make a default resetAction */
    private void makeResetAction() {
        ImageIcon icon = getIcon("Home");
        reset = new AbstractAction("Reset", icon) {
            public void actionPerformed(ActionEvent e) {
                if (display != null) {
                    display.getLayoutModel().reset();
                }
            }
        };
        actions.put("reset", reset);
    }
    
    /** Make a default moveLeftAction */
    private void makeMoveLeftAction() {
        ImageIcon icon = getIcon("Back");
        moveLeft = new AbstractAction("Move Left", icon) {
            public void actionPerformed(ActionEvent e) {
                if (display != null) {
                    display.getLayoutModel().moveLeft();
                }
            }
        };
        actions.put("moveLeft", moveLeft);
    }
    
    /** Make a default moveRightAction */
    private void makeMoveRightAction() {
        ImageIcon icon = getIcon("Forward");
        moveRight = new AbstractAction("Move Right", icon) {
            public void actionPerformed(ActionEvent e) {
                if (display != null) {
                    display.getLayoutModel().moveRight();
                }
            }
        };
        actions.put("moveRight", moveRight);
    }
    
    /** Make a default searchAction */
    private void makeSearchAction() {
        ImageIcon icon = getIcon("Search");
        search = new AbstractAction("Search", icon) {
            public void actionPerformed(ActionEvent e) {
                String key = JOptionPane.showInputDialog(display, "Search for ...", "Sequence Search", JOptionPane.QUESTION_MESSAGE);
                if ((key == null) || (key.length() == 0)) return ;
                display.search(key);
            }
        };
        actions.put("search", search);
    }
    
    /** Make a default help action */
    private void makeHelpAction() {
        ImageIcon icon = getIcon("Help");
        help = new AbstractAction("Help", icon) {
            public void actionPerformed(ActionEvent e) {
                System.out.println("SequenceActions.doHelp");
            }
        };
        actions.put("help", help);
    }
    
    /** Make a default quit action */
    private void makeExitAction() {
        exit = new AbstractAction("Exit") {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        };
        actions.put("exit", exit);
    }
    
    private ImageIcon getIcon(String name) {
        ImageIcon icon = null;
        String file = "image/" + name + iconSize + ".gif";
        if (!inApplet) icon = new ImageIcon(file);
        else {
            java.net.URL url = getClass().getResource(file);
            icon = new ImageIcon(url);
        }
        return icon;
    }
    
    /** Use big icons */
    public void setBigIcon() {
        iconSize = BIG_ICON;
        initialize();
    }
    
    /** Use small icons */
    public void setSmallIcon() {
        iconSize = SMALL_ICON;
        initialize();
    }
    
    /** Expose the actions set
     * @return the actions set
     */
    public Map getActions() {
        return actions;
    }
    
    /** Set the SequenceDisplay for all Actions.
    */
    public void setSequenceDisplay(SequenceDisplay dis) {
        display = dis;
    }
    
    /** Set zoomAction */
    public void setZoomAction(Action a) {
        this.zoom = a;
    }
    
    /** Get the zoomAction. */
    public Action getZoomAction() {
        return this.zoom;
    }
    
    /** Set zoomINActin */
    public void setZoomInAction(Action a) {
        this.zoomIn = a;
    }
    /** Get the zoomInAction. */
    public Action getZoomInAction() {
        return this.zoomIn;
    }
    
    /** Set zoomOUtAction */
    public void setZoomOutAction(Action a) {
        this.zoomOut = a;
    }
    /** Get the zoomOutAction. */
    public Action getZoomOutAction() {
        return this.zoomOut;
    }
    
    /** Set Reset Action */
    public void setResetAction(Action a) {
        this.reset = a;
    }
    
    /** Get the Reset Action. */
    public Action getResetAction() {
        return this.reset;
    }
    
    /** Set moveLeft action. */
    public void setMoveLeftAction(Action a) {
        this.moveLeft = a;
    }
    
    /** Get the moveLeft Action. */
    public Action getMoveLeftAction() {
        return this.moveLeft;
    }
    
    /** Set the moveRight action. */
    public void setMoveRightAction(Action a) {
        this.moveRight = a;
    }
    
    /** Get the moveRight action. */
    public Action getMoveRightAction() {
        return this.moveRight;
    }
    
    /** Set the Search Action */
    public void setSearchAction(Action a) {
        this.search = a;
    }
    
    /** Get the search Action. */
    public Action getSearchAction() {
        return this.search;
    }
    
    /** Set help action */
    public void setHelpAction(Action a) {
        this.help = a;
    }
    
    /** Get the help action. */
    public Action getHelpAction() {
        return this.help;
    }
    
    /** Set the exit action. */
    public void setExitAction(Action exit) {
       this.exit = exit;
    }
    
    /** Get the exit action. */
    public Action getExitAction() {
        return exit;
    }
    
    class PropertyChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            if (e.getPropertyName().equalsIgnoreCase("range")) {
                if (display != null) {
                    MapRange range = (MapRange) e.getNewValue();
                    display.setMapRange(range);
                }
            }
        }
    }
}