package jadex.swing.sequence;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import org.jdom.Element;

import jadex.swing.map.MapRange;

/** SequenceDisplay.java: Graphics Sequence display designed based on Swing MVC model
 * and renderer method.
 * @version 1.0
 * @author wgm
 */

public class SequenceDisplay extends JComponent implements Scrollable {
    private SequenceRenderer renderer = null; // The render for display individual sequence
    private GeneRenderer geneRenderer = null; // The geneRenderer for display a gene with several exons
    private SequenceModel model = null; // The model for storing and processing data
    private SequenceSelectionModel selectionModel = null; // The selectionModel for selection
    private SequenceLayoutModel layoutModel = null; // The layoutModel for all sequence layout and zoom processing
    
    private JPopupMenu popup = null; // A JPopup for this display
    
    /** Default constructor */
    public SequenceDisplay() {
    }
    
    /** Second constructor.
    * @param model the data model for this SequenceDisplay.
    */
    public SequenceDisplay(SequenceModel model) {
        setModel(model);
    }
    
    /** Set the sequence model.
    * @param model the data model for this SequenceDisplay.
    */
    public void setModel(SequenceModel model) {
        SequenceModel old = this.model;
        this.model = model;
        firePropertyChange("SequenceModel", old, this.model);
    }
    
    /** Get the sequence model.
    * @return the data model used in this SequenceDisplay.
    */
    public SequenceModel getModel() {
        return this.model;
    }
    
    /** Set the sequence renderer.
    * @param renderer the SequenceRenderer used for rendering Seqeunce objects in
    * this SequenceDisplay.
    */
    public void setRenderer(SequenceRenderer renderer) {
        this.renderer = renderer;
    }
    
    /** Get the sequence renderer. */
    public SequenceRenderer getRenderer() {
        return this.renderer;
    }
    
    /** Set gene renderer. */
    public void setGeneRenderer(GeneRenderer renderer) {
        this.geneRenderer = renderer;
    }
    
    /** Get the gene renderer. */
    public GeneRenderer getGeneRenderer() {
        return this.geneRenderer;
    }
    
    /** Set the SequenceUI.
    * @param ui the SequenceUI used in the SequenceDisplay.
    */
    public void setUI(SequenceUI ui) {
        super.setUI(ui);
    }
    
    /** Get the SequenceUI used in this SequenceDisplay. */
    public SequenceUI getUI() {
        return (SequenceUI) ui;
    }
    
    /** Create and set SequenceUI for this SequenceDisplay. This method should be
    * called explicitly for correctly displaying.
    */
    public void updateUI() {
        SequenceUI ui = new SequenceUI();
        setUI(ui);
        invalidate();
    }
    
    /** Clear up and force to release all resources. */
    public void clear() {
        super.removeAll();
        getUI().uninstallUI(this);
        model = null;
        selectionModel.clearSelection();
        selectionModel = null;
    }
    
    /** Set up a popup menu */
    public void setPopup(JPopupMenu pop) {
        add(pop);
        this.popup = pop;
    }
    
    /** Expose the popup to the classes at the same package */
    public JPopupMenu getPopup() {
        return this.popup;
    }
    
    /** Set the selection model */
    public void setSelectionModel(SequenceSelectionModel model) {
        selectionModel = model;
    }
    
    /** Get the selectionModel used in this SequenceDisplay. */
    public SequenceSelectionModel getSelectionModel() {
        return selectionModel;
    }
    
    /** Set the layoutModel */
    public void setLayoutModel(SequenceLayoutModel model) {
        this.layoutModel = model;
    }
    
    /** Get the layoutModel used in this SequenceDisplay. */
    public SequenceLayoutModel getLayoutModel() {
        return this.layoutModel;
    }
    
    /** Set one and only one selected Sequence element.
    * @param sequence the selected seqeunce element.
    */
    public void setSelectedSequence(Element sequence) {
        if (selectionModel != null) selectionModel.setSelectedElement(sequence);
    }
    
    /** Set a list of selected sequence elements.
    * @param list the selected seqeunce element list.
    */
    public void setSelectedSequences(java.util.List list) {
        if (selectionModel != null) selectionModel.setSelectedElements(list);
    }
    
    /** Get the selected sequence element.
    * @return the selected sequence element.
    */
    public Element getSelectedSequence() {
        if (selectionModel != null) return selectionModel.getSelectedElement();
        else return null;
    }
    
    /** Get the selected sequence elemnet list.
    * @return the list of the selected sequence elements.
    */
    public java.util.List getSelectedSequences() {
        if (selectionModel == null) return null;
        return selectionModel.getSelection();
    }
    
    /** Remove selected sequence. 
    * @param sequence the sequence element to be removed from the selection.
    */
    public void deSelectSequence(Element sequence) {
        if (selectionModel != null) selectionModel.deSelectSequence(sequence);
    }
    
    /** Clear all selected sequences. */
    public void clearSelection() {
        if (selectionModel != null) selectionModel.clearSelection();
    }
    
    
    /** Specify a maprange for this display 
     * @param range the maprange to be layouted
     */
    public void setMapRange(MapRange range) {
        getLayoutModel().setMapRange(range);
    }
    
    /** Get the current maprange
     * @return the current maprange
     */
    public MapRange getMapRange() {
        return getLayoutModel().getMapRange();
    }
    
    /** Set the mouseAt position in bp unit */
    public void setMouseAt(long p) {
        getLayoutModel().setMouseAt(p);
    }
    
    /** Get the mouse position in bp unit */
    public void getMouseAt() {
        getLayoutModel().getMouseAt();
    }
    
    /** Search a sequence element with the name of name.
    * @param name the searching key.
    */
    public void search(String name) {
        if (model == null) return;
        Element found = model.search(name);
        if (found == null) return ;
        java.util.List selection = selectionModel.getSelection();
        if (!selection.contains(found)) selection.add(found);
        else { // I want to put found at the last of selection, so tree will scroll it to visible
            selection.remove(found);
            selection.add(found);
        }
        setSelectedSequences(selection);
        setMapRange(jadex.xml.XMLUtilities.getMapRange(found, found.getNamespace()));
        //repaint();
    }
    
    /** Release proteced access to public, so its package class can call it.
     */
    public void firePropertyChange(String name, Object oldValue, Object newValue) {
        super.firePropertyChange(name, oldValue, newValue);
    }
    
    
    public static void main (String[] args) {
        if (args.length < 1) {
            System.out.println("Usage java SequenceDisplay xmlFile");
            System.exit(0);
        }
        //long start = System.currentTimeMillis();
        final JFrame f = new JFrame("Sequence Display");
        f.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                f.setVisible(false);
                f.dispose();
                System.exit(0);
            }
        });
        
        SequenceDisplay display = new SequenceDisplay();
        display.updateUI();
        f.getContentPane().add(new JScrollPane(display), "Center");
        f.setSize(1024, 350);
        f.setVisible(true);
        SequenceFetcher fetcher = new SequenceXMLFetcher(args[0]);
        SequenceModel model = new SequenceModel(fetcher);
        display.setModel(model);
        //long end = System.currentTimeMillis();
        //System.out.println("Time used: " + (end - start));
        display.search("C01F4");
    }
    
    // Scrollable methods
    /** Horizontal is always in a whole view */
    public boolean getScrollableTracksViewportHeight() {
        return false;
    }
    /** */
    public int getScrollableUnitIncrement(final java.awt.Rectangle p1,int p2,int p3) {
        if (p2 == SwingConstants.VERTICAL)  return (int) layoutModel.height;
        else return p3;
    }
    public int getScrollableBlockIncrement(final java.awt.Rectangle p1,int p2,int p3) {
        return (p2 == SwingConstants.VERTICAL) ? p1.height : p1.width;
    }
    public java.awt.Dimension getPreferredScrollableViewportSize() {
        return this.getPreferredSize();
    }
    public boolean getScrollableTracksViewportWidth() {
        return true;
    }
}