/*
 * SequenceTreeNode.java: A tree node is used in JTree for Sequence xml document
 *
 * Created on December 5, 2000, 1:58 PM
 */

package jadex.swing.sequence;

import org.jdom.Element;
import javax.swing.tree.*;
import java.util.*;

import jadex.xml.XMLUtilities;
import jadex.swing.map.MapRange;
import jadex.swing.map.UnknownPositionException;

/**
 * A customized TreeNode used in SequenceTreeDisplay by using JDOM element.
 * @author  wgm
 * @version 1.0
 */
public class SequenceTreeNode implements TreeNode {

    private Element sequence = null; // The wrapped sequence element. 
    private List list = null; // The list of children of the sequence element.

    // A comparator for sorting list alphabetically. This is default here.
    private Comparator posComp = new Comparator() {
        public int compare(Object o1, Object o2) {
            Element seq1 = (Element) o1;
            Element seq2 = (Element) o2;
            MapRange range1 = XMLUtilities.getMapRange(seq1, seq1.getNamespace());
            MapRange range2 = XMLUtilities.getMapRange(seq2, seq2.getNamespace());
            long pos1 = 0, pos2 = 0;
            try {
                pos1 = (range1.getStart() + range1.getEnd()) / 2;
                pos2 = (range2.getStart() + range2.getEnd()) / 2;
            }
            catch(UnknownPositionException e) {
                System.err.println("SequenceTreeNode.posComp: " + e);
            }

            int rtn = 0;
            if ((pos1 - pos2) < 0) rtn = -1;
            else if ((pos1 - pos2) > 0) rtn = 1;

            return rtn;
        }
    };

    private Comparator nameComp = new Comparator() {
        public int compare(Object obj1, Object obj2) {
            Element seq1 = (Element) obj1;
            Element seq2 = (Element) obj2;
            String name1 = seq1.getAttributeValue("name");
            String name2 = seq2.getAttributeValue("name");
            return name1.compareTo(name2);
        }
    };

    /** Creates new SequenceTreeNode */
    public SequenceTreeNode() {
    }

    /** Second construcor
     * @param seq the sequence element for the current treenode
     */
    public SequenceTreeNode(Element seq) {
        setSequence(seq);
    }

    /** Set the wrapped sequence element.
    * @param seq the wrapped sequence element.
    */
    public void setSequence(Element seq) {
        this.sequence = seq;
        if (seq != null) {
            list = jadex.xml.XMLUtilities.getSubSequences(sequence, sequence.getNamespace());
            if (list != null) Collections.sort(list, nameComp);
        }
    }

    /** Get the wrapped sequence element.
    * @return the wrapped sequence element.
    */
    public Element getSequence() {
        return this.sequence;
    }
    
    /** This method return the name of the sequence element.
    */
    public String toString() {
        return sequence.getAttributeValue("name");
    }

    /** Comparing if two elements are equal.
    */
    public boolean equals(Object o) {
        if ( o instanceof SequenceTreeNode) {
            SequenceTreeNode node = (SequenceTreeNode) o;
            if (this.sequence.equals(node.getSequence())) {
                //System.out.println("Node1: " + this.toString() + " Node2: " + o.toString() + " true");
                return true;
            }
            else {
                //System.out.println("Node1: " + this.toString() + " Node2: " + o.toString() + " false");
                return false;
            }
        }
        else return false;
    }

    /** Get the index of a child node.
    * @param p1 a child node.
    */
    public int getIndex(final javax.swing.tree.TreeNode p1) {
        if (list == null) return -1;
        SequenceTreeNode node = (SequenceTreeNode) p1;
        Element seq2 = node.getSequence();
        return list.indexOf(seq2);
    }
    
    /** Get if this node allows children.
    * @return always true.
    */
    public boolean getAllowsChildren() {
        return true;
    }
    
    /** Get a treeNode given the index of the child.
    * @param p1 the index of the child.
    */
    public javax.swing.tree.TreeNode getChildAt(int p1) {
        if (list == null) return null;
        Element seq = (Element) list.get(p1);
        return new SequenceTreeNode(seq);
    }
    
    /** Get the total number of the children.
    * @return the total number of the children.
    */
    public int getChildCount() {
        if (list == null) return -1;
        return list.size();
    }
    
    /** Get all children in an Enumeration.
    */
    public java.util.Enumeration children() {
        return Collections.enumeration(list);
    }
    
    /** See if this TreeNode is a leaf.
    */
    public boolean isLeaf() {
        if ((list == null) || (list.size() == 0)) return true;
        else return false;
    }
    
    /** Get the parent of this TreeNode.
    * @return the parent TreeNode of this TreeNode.
    */
    public javax.swing.tree.TreeNode getParent() {
        // Return null if it is the root in a xml document or there is no parent element.
        if (sequence.isRootElement() || (sequence.getParent() == null)) return null;
        Element parent = sequence.getParent().getParent();
        return new SequenceTreeNode(parent);
    }
}