/*
 * SequenceUI.java: A componentUI for SequenceDisplay
 *
 * Created on November 21, 2000, 2:24 PM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.beans.*;
import java.util.*;

import javax.swing.event.*;
import java.io.Serializable;
import javax.swing.*;
import javax.swing.plaf.*;

import org.jdom.*;

import jadex.swing.map.*;
import jadex.xml.XMLUtilities;

/**
 * A customized ComponentUI for processing SequenceDispay activities.
 * @author  wgm
 * @version 1.0
 */
public class SequenceUI extends ComponentUI implements Serializable {

    private CellRendererPane rendererPane = null; // The container all sequneces will be on
    private SequenceDisplay display = null; // The display to render all sequence objects

    private MouseInputListener mouseHandler = null; // All mouse activities processor
    private SelectionEventHandler selectionHandler = null; // Selection processor
    private PropertyChangeHandler propertyHandler = null; // PropertyChange handler
    
    private SequenceLayoutModel layoutModel = null; // Layour calculator

    private int currentLayer = 0; // The count of the layer of sequence element
    //private Namespace ns = null; // The namespace of the root

    private boolean doSelection = false; // A flag for selection repaint.
    private Rectangle paintBounds = null; // The current clipBounds of the graphics
       
    private Color connColor = new Color(204, 204, 255);
    
    /** Default constructor */
    public SequenceUI() {
    }

    /** Actually, an initializion method, following the model of ListUI */
    public void installUI(JComponent c) {
        display = (SequenceDisplay) c;
        rendererPane = new CellRendererPane();
        display.add(rendererPane);
        installDefaults();
        installListeners();
        // installKeyboardActions(); // Do not support keyboad activities now
    }

    /** initialize default settings */
    protected void installDefaults() {
        display.setLayout(null);
        display.setBorder(BorderFactory.createEtchedBorder());

        if (display.getRenderer() == null) {
            display.setRenderer(new SequenceRenderer());
        }
        
        if (display.getGeneRenderer() == null) {
            display.setGeneRenderer(new GeneRenderer());
        }

        if (display.getLayoutModel() == null) {
            layoutModel = new SequenceLayoutModel(display);
            display.setLayoutModel(layoutModel);
        }
        else layoutModel = display.getLayoutModel();
    }

    /** initialize listeners */
    protected void installListeners() {
        mouseHandler = new MouseEventHandler();

        display.addMouseListener(mouseHandler);
        display.addMouseMotionListener(mouseHandler);

        propertyHandler = new PropertyChangeHandler();
        display.addPropertyChangeListener(propertyHandler);

        selectionHandler = new SelectionEventHandler();
        SequenceSelectionModel selectionModel = display.getSelectionModel();
        if (selectionModel == null) {
            selectionModel = new SequenceSelectionModel();
            display.setSelectionModel(selectionModel);
        }
        selectionModel.addSelectionListener(selectionHandler);
    }

    /** removing all initializtion setting */
    public void uninstallUI(JComponent c) {
        uninstallDefaults();
        uninstallListeners();

        display.remove(rendererPane);
        rendererPane = null;
        display = null;
    }

    protected void uninstallDefaults() {
        if (display.getRenderer() != null) {
            display.setRenderer(null);
        }
        if (display.getGeneRenderer() != null) {
            display.setGeneRenderer(null);
        }
        if (display.getLayoutModel() != null) {
            display.setLayoutModel(null);
        }
    }

    protected void uninstallListeners() {
        display.removeMouseListener(mouseHandler);
        display.removeMouseMotionListener(mouseHandler);

        SequenceSelectionModel model = display.getSelectionModel();
        if (model != null) {
            model.removeSelectionListener(selectionHandler);
        }
    }

    /** Most of render activities are here */
    public void paint(Graphics g, JComponent c) {
        SequenceModel model = display.getModel();
        if (model == null) return;
        SequenceRenderer renderer = display.getRenderer();
        if (renderer == null) return;
        
        SequenceSelectionModel selectionModel = display.getSelectionModel();
        java.util.List list = selectionModel.getSelection(); // It will be safe here since selectionmodel is installed on initialization.

        // For a single sequence selection and this sequence is not a gene.
        if (doSelection) {
            doSelection = false;
            Element seq = selectionModel.getSelected();
            Component comp = renderer.getSequenceRenderer(seq, selectionHandler.isSelected(), false);
            if (comp == null) return ;
            int layer = XMLUtilities.getLayer(seq);
            boolean isOdd = XMLUtilities.isOdd(seq);
            Rectangle rect = layoutModel.getBounds(seq, layer, isOdd);
            if (rect == null) return;
            rendererPane.paintComponent(g, comp, display, rect.x, rect.y, rect.width, rect.height, true);    
            return;
        }
        
        
        layoutModel.relayout();
        
        // Only a clip region should be painted
        paintBounds = g.getClipBounds();
        
        // Set the background
        //g.setColor(Color.white);
        //g.fillRect(paintBounds.x, paintBounds.y, paintBounds.width, paintBounds.height);
        //long start = System.currentTimeMillis();
        paintRootSequence(g, renderer, list);
        //System.out.println("SequenceUI.paint: paintBounds: " + paintBounds.x + " " + paintBounds.y + " " + paintBounds.width + " " + paintBounds.height);
        //long end = System.currentTimeMillis();
        //System.out.println("Time in paint: " + (end - start));
        g.dispose();
    }
    
    private void paintRootSequence(Graphics g, SequenceRenderer renderer, java.util.List list1) {
        Element root = display.getModel().getRoot();
        if (root == null) return;
        paintSequence(g, renderer, root, 0, false, list1);
    }
    
    protected void paintSequence(Graphics g, SequenceRenderer renderer, Element sequence, java.util.List list1) {
        int layer = XMLUtilities.getLayer(sequence);
        boolean isOdd = XMLUtilities.isOdd(sequence);
        paintSequence(g, renderer, sequence, layer, isOdd, list1);
    }
    
    protected void paintSequence(Graphics g, SequenceRenderer renderer, Element sequence, int layer, boolean isOdd, java.util.List list1) {
        if (sequence == null) return;
        paintSingleSequence(g, renderer, sequence, layer, isOdd, list1);
        Element sub = sequence.getChild("Subsequence", sequence.getNamespace());
        if (sub == null) return;
        java.util.List list = sub.getChildren("Sequence", sequence.getNamespace());
        currentLayer ++;
        boolean odd = false; // The first sequence elemnet should be on the upper half of the box
        for (Iterator it = list.iterator(); it.hasNext();) {
            Element seq = (Element) it.next();
            //System.out.println("SequenceUI.paintSequence(): " + currentLayer);
            paintSequence(g, renderer, seq, currentLayer, odd, list1);
            odd = !odd;
        }
        currentLayer --;
    }
    
    protected void paintSingleSequence(Graphics g, SequenceRenderer renderer,
    Element sequence, int layer, boolean isOdd, java.util.List list1) {
        Rectangle rect = layoutModel.getBounds(sequence, layer, isOdd);
        if (rect == null) return ;
        if (rect.width < 1) rect.width = 1; // At least give something to see.
        
        g.setClip(rect);
        g.clipRect(paintBounds.x, paintBounds.y, paintBounds.width, paintBounds.height);
        if (g.getClipBounds().width == 0) return ; // No clip rigion, just stop
        Component comp = renderer.getSequenceRenderer(sequence, list1.contains(sequence), false);
        if (comp == null) return ;
        rendererPane.paintComponent(g, comp, display, rect.x, rect.y, rect.width, rect.height, true);
        // Paint the gene structures for those selected sequences
        if (list1.contains(sequence)) {
            paintGene(sequence, rect, g);
        }
    }
    
    /** Paint the structure of a gene 
     *@param seq a sequence element to be painted as a gene 
     */
    private void paintGene(Element seq, Rectangle rect1, Graphics g) {
        Element gene = seq.getChild("Gene", seq.getNamespace());
        if (gene == null) return ; // No gene, just return.
        MapRange range = XMLUtilities.getMapRange(seq, seq.getNamespace());
        long bp = 0;
        try { bp = range.getExtend(); }
        catch (UnknownPositionException e) {
            System.err.println("SequenceUI.paint(): " + e);
        }
        GeneRenderer geneRenderer = display.getGeneRenderer();
        Component comp = geneRenderer.getGeneRenderer(gene, display, bp);
        Rectangle rect = layoutModel.getGeneBounds(rect1);
        g.setClip(rect);
        g.clipRect(paintBounds.x, paintBounds.y, paintBounds.width, paintBounds.height);
        if (g.getClipBounds().width == 0) return;
        rendererPane.paintComponent(g, comp, display, rect.x, rect.y, rect.width, rect.height, true);
        // Draw a connecting line between gene rect and sequence rect
        int midX = (rect1.x + rect1.x + rect1.width) / 2;
        g.setClip(midX, rect.y + rect.height, 1, rect1.y - rect.y);
        g.setColor(connColor);
        g.drawLine(midX, rect1.y, midX, (rect.y + rect.height));
    }
    
    /** Paint a specfic sequence from outside of paint method, and is usually is called by
     * some event listeners.
     */
    protected void paintSelectedSequence(Element seq) {
        if (seq.getChild("Gene", seq.getNamespace()) != null) {
            //rect = layoutModel.getSequenceAndGeneBounds(seq, layer, isOdd);
            // If there is a gene selected, repaint the whole display. It should be fast to repaint
            // the whole display, since if a gene can be selected, there should not be a lot of
            // sequence elements to be repainted. This is not true is a gene can be selected by an external
            // mechanism, such as s SequenceTreeDisplay.
            doSelection = false;
            display.repaint();
            return;
        }
        int layer = XMLUtilities.getLayer(seq);
        boolean isOdd = XMLUtilities.isOdd(seq);
        Rectangle rect = layoutModel.getBounds(seq, layer, isOdd);
        if (rect == null) return;
        doSelection = true;
        display.repaint(rect.x, rect.y, rect.width, rect.height);
    }

    /** Handler to process sequence selection event */
    class SelectionEventHandler implements SequenceSelectionListener {
        private boolean isSelected;
        public void valueChanged(SequenceSelectionEvent e) {
            if (e.isMultiSelected()) {
                display.repaint(); // If this selection is a multiselection, just repaint display.
            }
            else {
                Element seq = e.getSequence();
                isSelected = e.isSelected();
                paintSelectedSequence(seq);
            }
        }
        public boolean isSelected() {
            return isSelected;
        }
    }

    /** Mouse event processing */
    class MouseEventHandler implements MouseInputListener {

        private boolean isZoom = false;
        private int inX, inY; // The starting coordinate of the mouse dragging
        private Graphics2D g2;
        private Rectangle zoom = null;
        
        public void mousePressed(final java.awt.event.MouseEvent p1) {
            if (p1.isPopupTrigger()) {
                if (display.getPopup() != null) {
                    display.getPopup().show(display, p1.getX(), p1.getY());
                }
            }
            else {
                if (!isZoom) {
                    inX = p1.getX();
                    inY = p1.getY();
                    isZoom = true;
                    zoom = null;
                    g2 = (Graphics2D) display.getGraphics();
                    g2.setColor(Color.red);
                    g2.fill(new Ellipse2D.Float(inX, inY, 1.0f, 1.0f));
                }
            }
        }
        public void mouseEntered(final java.awt.event.MouseEvent p1) {
        }
        public void mouseClicked(final java.awt.event.MouseEvent p1) {
            Element seq = layoutModel.getClosestElement(p1.getX(), p1.getY());
            
            if (seq == null) {
                if (zoom == null) return;
                if (zoom.contains(p1.getPoint())) {
                    layoutModel.zoom(zoom);
                }
                else {
                    // A little bigger than zoom to repaint the zooming region
                    display.repaint(zoom.x, zoom.y, zoom.width + 1, zoom.height + 1);
                }
                zoom = null;
                isZoom = false;
                return;
            }
            // First click to select. Click again to remove select.
            //if (p1.getClickCount() == 2) display.setSelectedSequence(seq);
            //else display.deSelectSequence(seq);
            if (display.getSelectedSequences().contains(seq)) {
                display.deSelectSequence(seq);
            }
            else {
                display.setSelectedSequence(seq);
            }
            isZoom = false;
        }
        public void mouseReleased(final java.awt.event.MouseEvent p1) {
            if (p1.isPopupTrigger()) {
                if (display.getPopup() != null) display.getPopup().show(display, p1.getX(), p1.getY());
                isZoom = false; // In Windows system, this is the popup trigger
            }
            else {
                if (isZoom && (zoom == null)) {
                    int outX = p1.getX();
                    int outY = p1.getY();
                    zoom = new Rectangle(inX, inY, (outX - inX), (outY - inY));
                    g2.draw(zoom);
                }
            }
        }
        public void mouseExited(final java.awt.event.MouseEvent p1) {
        }
        public void mouseDragged(final java.awt.event.MouseEvent p1) {
        }
        public void mouseMoved(final java.awt.event.MouseEvent p1) {
            layoutModel.setMouseAt(layoutModel.fromXtoBp(p1.getX()));
           // System.out.println("Mouse at: " + layoutModel.fromXtoBp(p1.getX()) + " bp");
        }
    }

    /** Property change listerner */
    class PropertyChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            String name = e.getPropertyName();
            if (name.equalsIgnoreCase("SequenceModel")) {
                SequenceModel model = (SequenceModel) e.getNewValue();
                Element root = model.getRoot();
                //ns = model.getNamespace();
                long[] pos = XMLUtilities.getPositions(root, root.getNamespace());
                display.getLayoutModel().setTotalBP(pos[1]);
            }
            else if (name.equalsIgnoreCase("range")) {
                display.setMapRange((MapRange) e.getNewValue());
            }
        }
    }
}