/*
 * SequenceViewFactory.java: This class is used to create a sequence view given 
 * a JFrame or a Applet
 *
 * Created on December 27, 2000, 11:45 AM
 */

package jadex.swing.sequence;

import java.awt.*;
import java.beans.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

import java.text.NumberFormat;

/**
 * This class is used to set up all GUIs for Sequence View in a JFrame or JApplet.
 * @author  wgm
 * @version 1.0
 */
public class SequenceViewFactory extends Object {
    
    private SequenceDisplay display = null; // The sequence display for a sequence object
    private HorizontalMapScale mapscale = null; // The mapscale to indicate the extend of the display
    private SequenceTreeDisplay treeDisplay = null; // The tree dislay of a sequence object
    private HorizontalMiniMapCartoon cartoon = null; // The minimapcartoon to show the displaying region on the whole chromosome
    private SequenceDisplayAndTreeAdapter adapter = null; // The adapter connecting SequenceDisplay and SequenceTreeDisplay.
    
    /** Frist constructor 
    * @param frame The parent container content
    */
    public SequenceViewFactory(JFrame frame) {
        initialize(frame);
    }

    /** Second constructor
    * @param applet the container for content
    */
    public SequenceViewFactory(final JApplet applet) {
        initialize(applet);
    }
    
    /** Helper to initialize all UIs
    */
    private void initialize(final JFrame frame) {
        frame.setTitle("Sequence View: ");
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                clear();
                frame.setVisible(false);
                frame.dispose();
            }
        });
        
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        JSplitPane sp = setUI(screenSize.width, screenSize.height - 30);
        
        SequenceActions actions = SequenceActions.getInstance();
        actions.setSequenceDisplay(display);
        JToolBar toolBar = setUpToolBar(actions);
        JMenuBar menubar = setUpMenuBar(actions);
        JPopupMenu popup = setUpPopup(actions);
             
        display.setPopup(popup);
        frame.getContentPane().add(toolBar, "North");
        frame.getContentPane().add(new JScrollPane(sp), "Center");
        frame.setJMenuBar(menubar);       
        frame.setSize(screenSize.width, screenSize.height - 30);  // Leave some space to taskbar on Windows
        frame.setVisible(true);  
    }
    
    private void initialize(JApplet applet) {
        JSplitPane sp = setUI(applet.getWidth(), applet.getHeight());
        
        SequenceActions actions = SequenceActions.getInstance(true);
        actions.setSequenceDisplay(display);
        JToolBar toolBar = setUpToolBar(actions);
        JPopupMenu popup = setUpPopup(actions);
        display.setPopup(popup);
        
        JPanel panel = new JPanel();
        panel.setBorder(BorderFactory.createEtchedBorder());
        panel.setLayout(new BorderLayout());
        panel.add(toolBar, BorderLayout.NORTH);
        panel.add(sp, BorderLayout.CENTER);
        applet.getContentPane().add(panel, BorderLayout.CENTER);
        //applet.getContentPane().add(sp, BorderLayout.CENTER);
    }
    
    /** Common ui setting */
    private JSplitPane setUI(int width, int height) {
        display = new SequenceDisplay();
        display.updateUI();
        mapscale = new HorizontalMapScale();
        display.addPropertyChangeListener(mapscale);
        // Add display and mapscale to the JFrame
        JPanel panel1 = new JPanel();
        panel1.setBorder(BorderFactory.createEtchedBorder());
        panel1.setLayout(new BorderLayout());
        panel1.add(display, "Center");
        panel1.add(mapscale, "South");
        mapscale.setPreferredSize(new Dimension(width - 24, 25));
        display.setPreferredSize(new Dimension(width - 24, 325));
        panel1.setMinimumSize(new Dimension(width - 24, 350));
        
        //Add TreeDisplay
        treeDisplay = new SequenceTreeDisplay();
        treeDisplay.setPreferredSize(new Dimension(width - 24, 250));
       
        JSplitPane sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        sp.setDividerSize(5);
        sp.add(panel1);
        sp.add(treeDisplay);
        
        return sp;
    }
    
    /** Set up a JToolBar for this view */
    private JToolBar setUpToolBar(SequenceActions actions) {
        JToolBar toolBar = new JToolBar();
        //actions.setBigIcon();
        toolBar.add(actions.getResetAction()).setToolTipText("Reset");
        toolBar.add(actions.getZoomAction()).setToolTipText("Zoom");
        toolBar.add(actions.getZoomInAction()).setToolTipText("Zoom In");
        toolBar.add(actions.getZoomOutAction()).setToolTipText("Zoom Out");
        toolBar.add(actions.getMoveLeftAction()).setToolTipText("Move Left");
        toolBar.add(actions.getMoveRightAction()).setToolTipText("Move Right");
        toolBar.addSeparator();
        toolBar.add(actions.getSearchAction()).setToolTipText("Search");
        // Add a minimapcartoon at the right end of the toolbar
        cartoon = new HorizontalMiniMapCartoon();
        display.addPropertyChangeListener(cartoon);
        // Add a Jabel to show the position of mouse in bp unit
        PositionLabel posLabel = new PositionLabel();
        display.addPropertyChangeListener(posLabel);
        
        JPanel cartoonP = new JPanel();
        cartoonP.setLayout(new FlowLayout(FlowLayout.RIGHT));
        cartoonP.add(posLabel);
        cartoonP.add(cartoon);
        toolBar.add(cartoonP);
        //toolBar.setBorderPainted(false);
        //toolBar.setBorder(BorderFactory.createEtchedBorder());
        return toolBar;
    }
    
    /** Set up a JMunubar for the sequence view */
    private JMenuBar setUpMenuBar(SequenceActions actions) {
        JMenuBar menubar = new JMenuBar();
        //JMenu file = new JMenu("File");
        //file.add(actions.getExitAction());
        
        JMenu view = new JMenu("View");
        view.add(actions.getZoomAction());
        view.add(actions.getZoomInAction());
        view.add(actions.getZoomOutAction());
        view.addSeparator();
        view.add(actions.getMoveLeftAction());
        view.add(actions.getMoveRightAction());
        view.addSeparator();
        view.add(actions.getResetAction());
        
        JMenu search = new JMenu("Search");
        search.add(actions.getSearchAction());
        
        JMenu help = new JMenu("Help");
        help.add(actions.getHelpAction());
        //menubar.add(file);
        menubar.add(view);
        menubar.add(search);
        menubar.add(help);   
        return menubar;
    }
    
    /** Set up a Jpopup */
    private JPopupMenu setUpPopup(SequenceActions actions) {
        JPopupMenu popup = new JPopupMenu();
        popup.add(actions.getZoomAction());
        popup.add(actions.getZoomInAction());
        popup.add(actions.getZoomOutAction());
        popup.addSeparator();
        popup.add(actions.getMoveLeftAction());
        popup.add(actions.getMoveRightAction());
        popup.addSeparator();
        popup.add(actions.getResetAction());
        return popup;
    }
    
    /** Set the sequence fetcher, i.e., data source, to be displayed
    * @param fetcer sequence data source name
    */
    public void setSequenceFetcher(SequenceFetcher fetcher) {
        SequenceModel model = new SequenceModel(fetcher);
        display.setModel(model);       
        treeDisplay.setRootElement(model.getRoot());       
        adapter = new SequenceDisplayAndTreeAdapter(display, treeDisplay);
        
        cartoon.setFullMapRange(display.getMapRange());
    }
    
    /** Search a sequence element in the display
     * @param seqeunce the sequence to be searched
     */
    public void search(String sequence) {
        if (display != null) display.search(sequence);
    }
    
    /** Clear up method.  */
    public void clear() {
        if (adapter != null) adapter.clear();
        if (display != null) display.clear();
        if (treeDisplay != null) treeDisplay.clear();
    }
     
    /** Get the SequenceDisplay in this object.
    * @return the SequenceDislay used in this object.
    */
    public SequenceDisplay getSequenceDisplay() {
        return this.display;
    }
    
    /** Get the SequenceTreeDisplay in this object.
    * @param the SequenceTreeDisplay used in this object.
    */
    public SequenceTreeDisplay getTreeDisplay() {
        return this.treeDisplay;
    }
    
    /** A customized to monitor the position of the Mouse in the SequenceDisplay. */
    class PositionLabel extends JLabel implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            if (e.getPropertyName().equalsIgnoreCase("mousePosition")) {
                Long pos = (Long)e.getNewValue();
                String posString = NumberFormat.getNumberInstance().format(pos.longValue());
                setText("Mouse at: " + posString + " bp");
            }
        }
    }
}