package jadex.swing.sequence;

/** SequenceXMLFetcher.java: a fetcher to load sequence data from an xml and talks to SequenceModel
 *
 * @version 1.0
 * @author wgm
 */
import java.util.*;

import org.jdom.*;
import org.jdom.input.*;
import java.io.*;

import jadex.xml.XMLUtilities;

/** 
 * A subclass SequenceFetcher to get data from a XML document.
 * @version 1.0
 * @author wgm
*/

public class SequenceXMLFetcher implements SequenceFetcher {
    
    private Document document = null; // The document loaded from a file source
    private Element root = null; // The root of the document
    
    /** Default constructor */
    public SequenceXMLFetcher() {
    }
    
    /** Second constructor 
     * @param file input file name
     */
    public SequenceXMLFetcher(String file) {
        FileInputStream source = null;
        try {
            source = new FileInputStream(file);
        }
        catch(IOException e) {
            System.err.println("SequenceXMLFetcher.init(): " + e);
        }
        setSource(source);
    }
    
    /** Third constructor: more general than the second one
     * @param input the source input stream
     */
    public SequenceXMLFetcher(InputStream input) {
        setSource(input);
    }
    
    /** Fourth constructor
    * @param input the data source
    & @param saxdriver the name of saxdriver 
    */
    public SequenceXMLFetcher(InputStream input, String saxdriver) {
        setSource(input, saxdriver);
    }
    
    /** Create a xml document based on an input stream. All xml processing activities
     * take place here. Pay specical attention here for the furture improvement. Now the defautl 
     * SAX parser, the Apache Xerces, is used based on the default setting in JDOM.
     * @param input document cource
     */
    public void setSource(InputStream input) {
        SAXBuilder builder = new SAXBuilder(false); // No validation is needed here
        try { document = builder.build(input); }
        catch(JDOMException e) { System.err.println("SequenceXMLFetcher.setSource(): " + e); }
        try {input.close();}
        catch(IOException e) {}
        root = document.getRootElement();
    }
    
    /** Overloaded method to set the source of xml.
    * @param input the source for the XML document.
    * @param saxdriver the SAX driver used for XML parser.
    */
    public void setSource(InputStream input, String saxdriver) {
        SAXBuilder builder = new SAXBuilder(saxdriver, false);
        try {
            document = builder.build(input);
        }
        catch(JDOMException e) {
            System.err.println("SequenceXMLFetcher.setSource(): " + e);
        }
        try {input.close();}
        catch(IOException e) {}
        root = document.getRootElement();
    }
    
    /** Get the root of the chromosome XML document.
     * @return the topmose element, i.e., Chromosome element
     */
    public Element getRoot() {
        // If root is chromsome and has no range element, try to construct one from its
        // subsequence
        if (root.getName().equalsIgnoreCase("Chromosome")) {
            Element range = null;
            Namespace ns = Namespace.getNamespace(root.getNamespaceURI());
            range = root.getChild("Range", ns);
            if (range == null) {
                Element sub = root.getChild("Subsequence", ns);
                List list = sub.getChildren("Sequence", ns);
                
                // Just get the end value of the last sequence, and this value is the same as the length
                // of the whole chromosome as long as these subsequences have already be sorted.
                Element last = (Element) list.get(list.size() - 1);
                long[] pos = XMLUtilities.getPositions(last, ns);
                range = new Element("Range", ns);
                range.addAttribute("start", "1");
                range.addAttribute("end", pos[1] + "");
                range.addAttribute("length", pos[1] + "");
                root.addContent(range);
            }
        }
        return root;
    }
    
    /** Get the XML document containing all Sequence element information.
    * @return document the xml document.
    */
    public Document getDocument() {
        return document;
    }
    /** The namespace of the root if any.
     */
    public Namespace getNamespace() {
        Namespace ns = null;
        if (root != null) {
            ns = Namespace.getNamespace(root.getNamespaceURI());
        }   
        return ns;
    }
    
    /** Set the XML document containing all sequence element information.
    * @param doc the XML document containing all sequence information.
    */
    public void setDocument(Document doc) {
        this.document = doc;
        this.root = document.getRootElement();
    }    
}