/*
 * Magnifier.java: A mouse listener and mousemotion listener to zoom a range in a JPanel
 *
 * Created on September 25, 2000, 2:09 PM
 */

package jadex.swing.util;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;

import javax.swing.*;
/**
 * A mouse listener and mousemotion listener to zoom a range in a JPanel. By using 
 * {@link Zoomable Zoomabl} together, a zoomable JPanel can be created. So-called zooming 
 * here means a region of map can be magnified by creating a rectangle.
 * @author  Guanming Wu
 * @version 1.0
 */
public class Magnifier implements MouseListener, MouseMotionListener {

    private Zoomable panel = null; // The panel to be zoomed
    private Rectangle2D.Float zoom = null; // The actual rectangle drawn by mouse
    private float inX, inY, outX, outY; // The four corner of the zoom
    private boolean doZooming = false; // A flag to show if zooming is in progress
    private boolean cursorChanged = false; // A flag to control the cursor change
    private float verticalPos; // Possible vertial position to be drawn around
    private Cursor magnifier = null; // If mouse is in a zoom range, change the cursor to this    
    
    /** constructor to initialize panel */
    public Magnifier(Zoomable p) { 
        setZoomable(p);
        initialize();
    }
    
    /** Set the Zoomable for zooming.
    * @param p a Zoomable can be magnified.
    */
    public void setZoomable(Zoomable p) { 
        this.panel = p; 
        p.addMouseListener(this);
        p.addMouseMotionListener(this);
    }

    // All initializing activities stuff
    private void initialize() {
        magnifier = createCursor();
        if (panel != null) verticalPos = panel.getVerticalPosition();
    }
    
    /** Set the vertical position. Only a rectangle around this position can be
    * used for zooming.
    * @param pos the vertical position.
    */
    public void setVerTicalPosition(float pos) {
        this.verticalPos = pos;
    }
    
    // A local helper method to create a customized cursor
    private Cursor createCursor() {
        Cursor c = null;
        try {
            Toolkit kit = Toolkit.getDefaultToolkit();
            String sep = File.separator;
            final String path = new String("jadex" + sep + "swing" + sep + "map" + sep);
            ImageIcon icon = new ImageIcon(path + "ZoomIn.gif");
            Image img = icon.getImage();

            c = kit.createCustomCursor(img, new Point(8, 8), "CUSTOMER_CURSOR");
            //System.out.println(kit.getBestCursorSize(4, 4));
        }
        catch (Exception e) {
            //System.err.println(e);
            c = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
        }
        return c;
    }
    
    /** Do nothing. */
    public void mouseReleased(final java.awt.event.MouseEvent p1) {
    }
    /** Do nothing. */
    public void mouseEntered(final java.awt.event.MouseEvent p1) {
    }
    /** Zooming in the drawn rectangle by clicking mouse.
    */
    public void mouseClicked(final java.awt.event.MouseEvent e) {
        if (doZooming) {
           if ((zoom.contains(e.getPoint())) && (outX > (verticalPos + 5.0f))) {
                panel.doZooming();  // Hope that panel will do something to clear up
                panel.setCursor(Cursor.getDefaultCursor());
                cursorChanged = false;
            }
            else {
                zoom = null;
                panel.setZoom(zoom);
                panel.repaint();
            }
            doZooming = false;
        }
    }
    /** Do nothing. */
    public void mousePressed(final java.awt.event.MouseEvent p1) {
    }
    /** Do nothing. */
    public void mouseExited(final java.awt.event.MouseEvent p1) {
    }
    
    /** Create a rectangle by mouse dragging. */
    public void mouseDragged(final java.awt.event.MouseEvent e) {
        if (!doZooming) {
            if (e.getX() > (verticalPos + 5.0f)) return;
            inX = e.getX();
            inY = e.getY();
            doZooming = true;
            zoom = new Rectangle2D.Float();
            return;
        }
        outX = e.getX();
        outY = e.getY();
        zoom.setRect(inX, inY, outX - inX, outY - inY);
        panel.setZoom(zoom);
        panel.repaint();
   }
    
   /** Detecting is mouse is in the drawn rectangle by mouse moving. If in, the cursor
    * will be changed to indicate a region can be zoomed.
    */
    public void mouseMoved(final java.awt.event.MouseEvent e) {
         if (doZooming) {
            if ((zoom.contains(e.getPoint())) && (outX > (verticalPos + 5.0f))) {
                if (!cursorChanged) {
                    panel.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                    //panel.setCursor(magnifier);
                    cursorChanged = true;
                }
            }
            else if (cursorChanged) {
                panel.setCursor(Cursor.getDefaultCursor());
                cursorChanged = false;
            }
        }
        else if (cursorChanged) {
            panel.setCursor(Cursor.getDefaultCursor());
            cursorChanged = false;
        }
    }
}