/*
 * XMLUtilities.java: Put some frequently used methods here for convience
 *
 * Created on November 28, 2000, 4:06 PM
 */

package jadex.xml;

import java.util.*;

import org.jdom.*;

import jadex.swing.map.MapRange;

/**
 * This is a utility class for processing XML files in Sequence View. Some frequently
 * used methods for XML processing are here. All these methods are static to keep methods
 * calling simple. The element data structure used here is JDOM format.
 * @author  wgm
 * @version 1.0
 */
public class XMLUtilities extends Object {

    /** Get the Namespace. In this Sequence View, it always 
    * return "http://www.wormbase.org". However, it should not be like this.
    * @return the namespace used in our sequenceview. Always for the wormbase.
    * @deprecated please don't use this method anymore.
    */
    public static Namespace getNamespace() {
        return Namespace.getNamespace("http://www.wormbase.org");
    }
    
    /** Get the position of the sequence element.
    * @param sequence the sequence element to be parsed.
    * @param ns the Namespace this sequence elemenet used.
    * @return a two element array: the first is the start postion, and the second the end position.
    * @deprecate use @getPositions(Element).
    */
    public static long[] getPositions(Element sequence, Namespace ns) {
        Element range = sequence.getChild("Range", sequence.getNamespace());
        if (range == null) return null;
        return parseRange(range);
    }

    /** Get the position of the sequence element.
    * @param sequence the sequenc elemnet too be parsed.
    * @return a two element array: the first is the start position , and the second the end position.
    */
    public static long[] getPositions(Element sequence) {
        Element range = sequence.getChild("Range", sequence.getNamespace());
        if (range == null) return null;
        return parseRange(range);
    }

    /** Extract the maprange information from sequence element
     * @param sequence element
     * @ns namespace for sequence element
     * @return the maprange for this sequence
     */
    public static MapRange getMapRange(Element sequence, Namespace ns) {
        long[] pos = getPositions(sequence, ns);
        return new MapRange(pos[0], pos[1]);
    }

    /** Extract the maprange information from sequence element
     * @param sequence element
     * @return the maprange for this sequence
     */
    public static MapRange getMapRange(Element sequence) {
        long[] pos = getPositions(sequence);
        return new MapRange(pos[0], pos[1]);
    }

    /** A helper to parse the Range element.
    */
    private static long[] parseRange(Element range) {
        long[] pos = new long[2];
        Attribute startA = range.getAttribute("start");
        Attribute endA = range.getAttribute("end");
        String startS = startA.getValue();
        String endS = endA.getValue();
        pos[0] = Long.parseLong(startS);
        pos[1] = Long.parseLong(endS);
        return pos;
    }

    /** Get the subsequences from a sequence element or a chromosome (root).
    * @param sequence the element to be parsed.
    * @return the subsequence list in this sequence element.
    */
    public static List getSubSequences(Element sequence) {
        Element sub = sequence.getChild("Subsequence");
        if (sub == null) return null;
        else return sub.getChildren("Sequence");
    }

    /** Get the subsequences from a sequenc element.
    * @param sequence the sequence element to be parsed.
    * @param ns the name space this sequence element is in.
    * @return the subsequence list.
    */
    public static List getSubSequences(Element sequence, Namespace ns) {
        Element sub = sequence.getChild("Subsequence", ns);
        if (sub == null) return null;
        else return sub.getChildren("Sequence", ns);
    }

    /** Get the name of the sequence from its attribute.
    * @param sequence the sequenc element.
    * @return the sequence name.
    */
    public static String getName(Element sequence) {
        Attribute name = sequence.getAttribute("name");
        return name.getValue();
    }

    /** Get the layer of a element in XML document from root element, i.e., Chromosome.
    * @param sequence the sequence element to be analyzed.
    * @return the layer of this sequence element.
    */
    public static int getLayer(Element sequence) {
        int layer = 0;
        Element element = sequence;
        while (true) {
            if (element.isRootElement()) break;
            element = element.getParent();
            if (element == null) break; // No more parent, topmost element has been reached.
            layer ++;
        }
        return layer / 2; // have to consider the sub layer
    }
    
    /** See the position of the sequence element in the subsequenc list is Odd or even.
    * @param sequence the sequence element to be analyzed.
    * @return true if odd, e.g., 0, 2, 4, etc. Otherwise false, e.g., 1, 3, 5, ...
    */
    public static boolean isOdd(Element sequence) {
        if (sequence.isRootElement() || (sequence.getParent() == null)) return false;
        Element parent = sequence.getParent();
        List list = parent.getChildren("Sequence", sequence.getNamespace());
        int index = list.indexOf(sequence);
        if ((index % 2) == 0) return false;
        else return true;
    }
}